# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import List, Optional

from stytch.consumer.models.connected_apps import ConnectedAppPublic
from stytch.consumer.models.idp import ScopeResult
from stytch.consumer.models.users import User
from stytch.core.response_base import ResponseBase


class AuthorizeResponse(ResponseBase):
    """Response type for `OAuth.authorize`.
    Fields:
      - redirect_uri: The callback URI used to redirect the user after authentication. This is the same URI provided at the start of the OAuth flow.  This field is required when using the `authorization_code` grant.
      - authorization_code: A one-time use code that can be exchanged for tokens.
    """  # noqa

    redirect_uri: str
    authorization_code: Optional[str] = None


class AuthorizeStartResponse(ResponseBase):
    """Response type for `OAuth.authorize_start`.
    Fields:
      - user_id: The unique ID of the affected User.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
      - client: (no documentation yet)
      - consent_required: Whether the user must provide explicit consent for the authorization request.
      - scope_results: Details about each requested scope.
    """  # noqa

    user_id: str
    user: User
    client: ConnectedAppPublic
    consent_required: bool
    scope_results: List[ScopeResult]



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Any, Dict, Optional

import jwt

from stytch.consumer.api.idp_oauth import OAuth
from stytch.consumer.api.policy_cache import PolicyCache
from stytch.consumer.models.idp import IDPTokenClaims, IDPTokenResponse
from stytch.consumer.models.sessions import AuthorizationCheck
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient
from stytch.shared import jwt_helpers, rbac_local


class IDP:
    def __init__(
        self,
        api_base: ApiBase,
        sync_client: SyncClient,
        async_client: AsyncClient,
        jwks_client: jwt.PyJWKClient,
        project_id: str,
        policy_cache: PolicyCache,
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client
        self.policy_cache = policy_cache
        self.jwks_client = jwks_client
        self.project_id = project_id
        self.oauth = OAuth(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )

    # MANUAL(IDP)(SERVICE_METHOD)
    # ADDIMPORT: from stytch.shared import jwt_helpers, rbac_local
    # ADDIMPORT: from stytch.consumer.models.idp import IDPTokenClaims, IDPTokenResponse
    # ADDIMPORT: from stytch.consumer.models.sessions import AuthorizationCheck
    def introspect_token_network(
        self,
        token: str,
        client_id: str,
        client_secret: Optional[str] = None,
        token_type_hint: str = "access_token",
        authorization_check: Optional[AuthorizationCheck] = None,
    ) -> Optional[IDPTokenClaims]:
        """Introspects a token JWT from an authorization code response.
        Access tokens are JWTs signed with the project's JWKs. Refresh tokens are opaque tokens.
        Access tokens contain a standard set of claims as well as any custom claims generated from templates.

        Fields:
          - token: The access token (or refresh token) to introspect.
          - client_id: The ID of the client.
          - client_secret: The secret of the client.
          - token_type_hint: A hint on what the token contains. Valid fields are 'access_token' and 'refresh_token'.
          - authorization_check: An optional authorization check to perform on the token.
        """
        headers: Dict[str, str] = {"Content-Type": "application/x-www-form-urlencoded"}
        data: Dict[str, Any] = {
            "token": token,
            "client_id": client_id,
            "token_type_hint": token_type_hint,
        }
        if client_secret is not None:
            data["client_secret"] = client_secret

        url = self.api_base.url_for(
            f"/v1/public/{self.project_id}/oauth2/introspect", data
        )
        res = self.sync_client.post_form(url, data, headers)
        jwtResponse = IDPTokenResponse.from_json(res.response.status_code, res.json)
        custom_claims = {
            k: v for k, v in res.json.items() if k not in self._non_custom_claim_keys()
        }
        if not jwtResponse.active:
            return None

        if authorization_check is not None:
            rbac_local.perform_consumer_scope_authorization_check(
                policy=self.policy_cache.get(),
                token_scopes=jwtResponse.scope.split(),
                authorization_check=authorization_check,
            )

        return IDPTokenClaims(
            subject=jwtResponse.sub,
            scope=jwtResponse.scope,
            audience=jwtResponse.aud,
            expires_at=jwtResponse.exp,
            issued_at=jwtResponse.iat,
            issuer=jwtResponse.iss,
            not_before=jwtResponse.nbf,
            token_type=jwtResponse.token_type,
            custom_claims=custom_claims,
        )

    async def introspect_token_network_async(
        self,
        token: str,
        client_id: str,
        client_secret: Optional[str] = None,
        token_type_hint: str = "access_token",
        authorization_check: Optional[AuthorizationCheck] = None,
    ) -> Optional[IDPTokenClaims]:
        """Introspects a token JWT from an authorization code response.
        Access tokens are JWTs signed with the project's JWKs. Refresh tokens are opaque tokens.
        Access tokens contain a standard set of claims as well as any custom claims generated from templates.

        Fields:
          - token: The access token (or refresh token) to introspect.
          - client_id: The ID of the client.
          - client_secret: The secret of the client.
          - token_type_hint: A hint on what the token contains. Valid fields are 'access_token' and 'refresh_token'.
        """
        headers: Dict[str, str] = {"Content-Type": "application/x-www-form-urlencoded"}
        data: Dict[str, Any] = {
            "token": token,
            "client_id": client_id,
            "token_type_hint": token_type_hint,
        }
        if client_secret is not None:
            data["client_secret"] = client_secret

        url = self.api_base.url_for(
            f"/v1/public/{self.project_id}/oauth2/introspect", data
        )
        res = await self.async_client.post_form(url, data, headers)
        jwtResponse = IDPTokenResponse.from_json(res.response.status, res.json)
        custom_claims = {
            k: v for k, v in res.json.items() if k not in self._non_custom_claim_keys()
        }
        if not jwtResponse.active:
            return None

        if authorization_check is not None:
            rbac_local.perform_consumer_scope_authorization_check(
                policy=self.policy_cache.get(),
                token_scopes=jwtResponse.scope.split(),
                authorization_check=authorization_check,
            )

        return IDPTokenClaims(
            subject=jwtResponse.sub,
            scope=jwtResponse.scope,
            audience=jwtResponse.aud,
            expires_at=jwtResponse.exp,
            issued_at=jwtResponse.iat,
            issuer=jwtResponse.iss,
            not_before=jwtResponse.nbf,
            token_type=jwtResponse.token_type,
            custom_claims=custom_claims,
        )

    def introspect_access_token_local(
        self,
        access_token: str,
        authorization_check: Optional[AuthorizationCheck] = None,
    ) -> Optional[IDPTokenClaims]:
        """Introspects a token JWT from an authorization code response.
        Access tokens are JWTs signed with the project's JWKs. Refresh tokens are opaque tokens.
        Access tokens contain a standard set of claims as well as any custom claims generated from templates.

        Fields:
          - access_token: The access token (or refresh token) to introspect.
          - client_id: The ID of the client.
          - authorization_check: An optional authorization check to perform on the token.
        """
        _scope_claim = "scope"
        generic_claims = jwt_helpers.authenticate_jwt_local(
            project_id=self.project_id,
            jwks_client=self.jwks_client,
            jwt=access_token,
            base_url=self.api_base.base_url,
        )
        if generic_claims is None:
            return None

        custom_claims = {
            k: v for k, v in generic_claims.untyped_claims.items() if k != _scope_claim
        }

        scope = generic_claims.untyped_claims[_scope_claim]

        if authorization_check is not None:
            rbac_local.perform_consumer_scope_authorization_check(
                policy=self.policy_cache.get(),
                token_scopes=scope.split(),
                authorization_check=authorization_check,
            )

        return IDPTokenClaims(
            subject=generic_claims.reserved_claims["sub"],
            scope=scope,
            custom_claims=custom_claims,
            audience=generic_claims.reserved_claims["aud"],
            expires_at=generic_claims.reserved_claims["exp"],
            issued_at=generic_claims.reserved_claims["iat"],
            issuer=generic_claims.reserved_claims["iss"],
            not_before=generic_claims.reserved_claims["nbf"],
            token_type="access_token",
        )

    def _non_custom_claim_keys(self):
        return [
            "aud",
            "exp",
            "iat",
            "iss",
            "jti",
            "nbf",
            "sub",
            "active",
            "client_id",
            "request_id",
            "scope",
            "status_code",
            "token_type",
        ]

    # ENDMANUAL(IDP)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from stytch.core.response_base import ResponseBase


class WhoamiResponse(ResponseBase):
    project_id: str
    name: str



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from stytch.b2b.models.rbac import OrgPolicy
from stytch.core.response_base import ResponseBase


class GetOrgPolicyResponse(ResponseBase):
    """Response type for `Organizations.get_org_policy`.
    Fields:
      - org_policy: The organization-specific RBAC Policy that contains roles defined for this organization. Organization policies supplement the project-level RBAC policy with additional roles that are specific to the organization.
    """  # noqa

    org_policy: OrgPolicy


class SetOrgPolicyResponse(ResponseBase):
    """Response type for `Organizations.set_org_policy`.
    Fields:
      - org_policy: The organization-specific RBAC Policy that contains roles defined for this organization. Organization policies supplement the project-level RBAC policy with additional roles that are specific to the organization.
    """  # noqa

    org_policy: OrgPolicy



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Optional

import pydantic


class MemberOptions(pydantic.BaseModel):
    """
    Fields:
      - mfa_phone_number: The Member's MFA phone number.
      - totp_registration_id: The Member's MFA TOTP registration ID.
    """  # noqa

    mfa_phone_number: str
    totp_registration_id: str


class MfaRequired(pydantic.BaseModel):
    """
    Fields:
      - member_options: Information about the Member's options for completing MFA.
      - secondary_auth_initiated: If null, indicates that no secondary authentication has been initiated. If equal to "sms_otp", indicates that the Member has a phone number, and a one time passcode has been sent to the Member's phone number. No secondary authentication will be initiated during calls to the discovery authenticate or list organizations endpoints, even if the Member has a phone number.
    """  # noqa

    member_options: Optional[MemberOptions] = None
    secondary_auth_initiated: Optional[str] = None



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Any, Dict, Optional, Union

from stytch.b2b.api.otp_email_discovery import Discovery
from stytch.b2b.models.otp_email import (
    AuthenticateRequestLocale,
    AuthenticateResponse,
    LoginOrSignupRequestLocale,
    LoginOrSignupResponse,
)
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class Email:
    def __init__(
        self, api_base: ApiBase, sync_client: SyncClient, async_client: AsyncClient
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client
        self.discovery = Discovery(
            api_base=self.api_base,
            sync_client=self.sync_client,
            async_client=self.async_client,
        )

    def login_or_signup(
        self,
        organization_id: str,
        email_address: str,
        login_template_id: Optional[str] = None,
        signup_template_id: Optional[str] = None,
        locale: Optional[Union[LoginOrSignupRequestLocale, str]] = None,
        login_expiration_minutes: Optional[int] = None,
        signup_expiration_minutes: Optional[int] = None,
    ) -> LoginOrSignupResponse:
        """Send either a login or signup email OTP to a Member. A new, pending, or invited Member will receive a signup email OTP. Non-active members will have a pending status until they successfully authenticate. An active Member will receive a login email OTP.

        The OTP is valid for 10 minutes. Only the most recently sent OTP is valid: when an OTP is sent, all OTPs previously sent to the same email address are invalidated, even if unused or unexpired.

        Fields:
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - email_address: The email address of the Member.
          - login_template_id: Use a custom template for login emails. By default, it will use your default email template. The template must be a template using our built-in customizations or a custom HTML email for OTP - Login.
          - signup_template_id: Use a custom template for signup emails. By default, it will use your default email template. The template must be a template using our built-in customizations or a custom HTML email for OTP - Signup.
          - locale: Used to determine which language to use when sending the user this delivery method. Parameter is a [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"`.

        Currently supported languages are English (`"en"`), Spanish (`"es"`), French (`"fr"`) and Brazilian Portuguese (`"pt-br"`); if no value is provided, the copy defaults to English.

        Request support for additional languages [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link")!

          - login_expiration_minutes: The expiration time, in minutes, for a login OTP email to a Member. If not authenticated within this time frame, the OTP will need to be resent. Defaults to 10 with a minimum of 2 and a maximum of 15.
          - signup_expiration_minutes: The expiration time, in minutes, for a signup OTP email to a Member. If not authenticated within this time frame, the OTP will need to be resent. Defaults to 10 with a minimum of 2 and a maximum of 15.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "organization_id": organization_id,
            "email_address": email_address,
        }
        if login_template_id is not None:
            data["login_template_id"] = login_template_id
        if signup_template_id is not None:
            data["signup_template_id"] = signup_template_id
        if locale is not None:
            data["locale"] = locale
        if login_expiration_minutes is not None:
            data["login_expiration_minutes"] = login_expiration_minutes
        if signup_expiration_minutes is not None:
            data["signup_expiration_minutes"] = signup_expiration_minutes

        url = self.api_base.url_for("/v1/b2b/otps/email/login_or_signup", data)
        res = self.sync_client.post(url, data, headers)
        return LoginOrSignupResponse.from_json(res.response.status_code, res.json)

    async def login_or_signup_async(
        self,
        organization_id: str,
        email_address: str,
        login_template_id: Optional[str] = None,
        signup_template_id: Optional[str] = None,
        locale: Optional[LoginOrSignupRequestLocale] = None,
        login_expiration_minutes: Optional[int] = None,
        signup_expiration_minutes: Optional[int] = None,
    ) -> LoginOrSignupResponse:
        """Send either a login or signup email OTP to a Member. A new, pending, or invited Member will receive a signup email OTP. Non-active members will have a pending status until they successfully authenticate. An active Member will receive a login email OTP.

        The OTP is valid for 10 minutes. Only the most recently sent OTP is valid: when an OTP is sent, all OTPs previously sent to the same email address are invalidated, even if unused or unexpired.

        Fields:
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - email_address: The email address of the Member.
          - login_template_id: Use a custom template for login emails. By default, it will use your default email template. The template must be a template using our built-in customizations or a custom HTML email for OTP - Login.
          - signup_template_id: Use a custom template for signup emails. By default, it will use your default email template. The template must be a template using our built-in customizations or a custom HTML email for OTP - Signup.
          - locale: Used to determine which language to use when sending the user this delivery method. Parameter is a [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"`.

        Currently supported languages are English (`"en"`), Spanish (`"es"`), French (`"fr"`) and Brazilian Portuguese (`"pt-br"`); if no value is provided, the copy defaults to English.

        Request support for additional languages [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link")!

          - login_expiration_minutes: The expiration time, in minutes, for a login OTP email to a Member. If not authenticated within this time frame, the OTP will need to be resent. Defaults to 10 with a minimum of 2 and a maximum of 15.
          - signup_expiration_minutes: The expiration time, in minutes, for a signup OTP email to a Member. If not authenticated within this time frame, the OTP will need to be resent. Defaults to 10 with a minimum of 2 and a maximum of 15.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "organization_id": organization_id,
            "email_address": email_address,
        }
        if login_template_id is not None:
            data["login_template_id"] = login_template_id
        if signup_template_id is not None:
            data["signup_template_id"] = signup_template_id
        if locale is not None:
            data["locale"] = locale
        if login_expiration_minutes is not None:
            data["login_expiration_minutes"] = login_expiration_minutes
        if signup_expiration_minutes is not None:
            data["signup_expiration_minutes"] = signup_expiration_minutes

        url = self.api_base.url_for("/v1/b2b/otps/email/login_or_signup", data)
        res = await self.async_client.post(url, data, headers)
        return LoginOrSignupResponse.from_json(res.response.status, res.json)

    def authenticate(
        self,
        organization_id: str,
        email_address: str,
        code: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        intermediate_session_token: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
        locale: Optional[Union[AuthenticateRequestLocale, str]] = None,
        telemetry_id: Optional[str] = None,
    ) -> AuthenticateResponse:
        """Authenticate a Member with a one-time passcode (OTP). This endpoint requires an OTP that is not expired or previously used.
        OTPs have a default expiry of 10 minutes. If the Member's status is `pending` or `invited`, they will be updated to `active`.
        Provide the `session_duration_minutes` parameter to set the lifetime of the session. If the `session_duration_minutes` parameter is not specified, a Stytch session will be created with a 60 minute duration.

        If the Member is required to complete MFA to log in to the Organization, the returned value of `member_authenticated` will be `false`, and an `intermediate_session_token` will be returned.
        The `intermediate_session_token` can be passed into the [OTP SMS Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-otp-sms), [TOTP Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-totp),
        or [Recovery Codes Recover endpoint](https://stytch.com/docs/b2b/api/recovery-codes-recover) to complete the MFA step and acquire a full member session.
        The `intermediate_session_token` can also be used with the [Exchange Intermediate Session endpoint](https://stytch.com/docs/b2b/api/exchange-intermediate-session) or the [Create Organization via Discovery endpoint](https://stytch.com/docs/b2b/api/create-organization-via-discovery) to join a different Organization or create a new one.
        The `session_duration_minutes` and `session_custom_claims` parameters will be ignored.

        If a valid `session_token` or `session_jwt` is passed in, the Member will not be required to complete an MFA step.

        Fields:
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - email_address: The email address of the Member.
          - code: The code to authenticate.
          - session_token: A secret token for a given Stytch Session.
          - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
          - intermediate_session_token: The Intermediate Session Token. This token does not necessarily belong to a specific instance of a Member, but represents a bag of factors that may be converted to a member session. The token can be used with the [OTP SMS Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-otp-sms), [TOTP Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-totp), or [Recovery Codes Recover endpoint](https://stytch.com/docs/b2b/api/recovery-codes-recover) to complete an MFA flow and log in to the Organization. The token has a default expiry of 10 minutes. It can also be used with the [Exchange Intermediate Session endpoint](https://stytch.com/docs/b2b/api/exchange-intermediate-session) to join a specific Organization that allows the factors represented by the intermediate session token; or the [Create Organization via Discovery endpoint](https://stytch.com/docs/b2b/api/create-organization-via-discovery) to create a new Organization and Member. Intermediate Session Tokens have a default expiry of 10 minutes.
          - session_duration_minutes: Set the session lifetime to be this many minutes from now. This will start a new session if one doesn't already exist,
          returning both an opaque `session_token` and `session_jwt` for this session. Remember that the `session_jwt` will have a fixed lifetime of
          five minutes regardless of the underlying session duration, and will need to be refreshed over time.

          This value must be a minimum of 5 and a maximum of 527040 minutes (366 days).

          If a `session_token` or `session_jwt` is provided then a successful authentication will continue to extend the session this many minutes.

          If the `session_duration_minutes` parameter is not specified, a Stytch session will be created with a 60 minute duration. If you don't want
          to use the Stytch session product, you can ignore the session fields in the response.
          - session_custom_claims: Add a custom claims map to the Session being authenticated. Claims are only created if a Session is initialized by providing a value in
          `session_duration_minutes`. Claims will be included on the Session object and in the JWT. To update a key in an existing Session, supply a new value. To
          delete a key, supply a null value. Custom claims made with reserved claims (`iss`, `sub`, `aud`, `exp`, `nbf`, `iat`, `jti`) will be ignored.
          Total custom claims size cannot exceed four kilobytes.
          - locale: Used to determine which language to use when sending the user this delivery method. Parameter is a [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"`.

        Currently supported languages are English (`"en"`), Spanish (`"es"`), French (`"fr"`) and Brazilian Portuguese (`"pt-br"`); if no value is provided, the copy defaults to English.

        Request support for additional languages [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link")!

          - telemetry_id: If the `telemetry_id` is passed, as part of this request, Stytch will call the [Fingerprint Lookup API](https://stytch.com/docs/fraud/api/fingerprint-lookup) and store the associated fingerprints and IPGEO information for the Member. Your workspace must be enabled for Device Fingerprinting to use this feature.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "organization_id": organization_id,
            "email_address": email_address,
            "code": code,
        }
        if session_token is not None:
            data["session_token"] = session_token
        if session_jwt is not None:
            data["session_jwt"] = session_jwt
        if intermediate_session_token is not None:
            data["intermediate_session_token"] = intermediate_session_token
        if session_duration_minutes is not None:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            data["session_custom_claims"] = session_custom_claims
        if locale is not None:
            data["locale"] = locale
        if telemetry_id is not None:
            data["telemetry_id"] = telemetry_id

        url = self.api_base.url_for("/v1/b2b/otps/email/authenticate", data)
        res = self.sync_client.post(url, data, headers)
        return AuthenticateResponse.from_json(res.response.status_code, res.json)

    async def authenticate_async(
        self,
        organization_id: str,
        email_address: str,
        code: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        intermediate_session_token: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
        locale: Optional[AuthenticateRequestLocale] = None,
        telemetry_id: Optional[str] = None,
    ) -> AuthenticateResponse:
        """Authenticate a Member with a one-time passcode (OTP). This endpoint requires an OTP that is not expired or previously used.
        OTPs have a default expiry of 10 minutes. If the Member's status is `pending` or `invited`, they will be updated to `active`.
        Provide the `session_duration_minutes` parameter to set the lifetime of the session. If the `session_duration_minutes` parameter is not specified, a Stytch session will be created with a 60 minute duration.

        If the Member is required to complete MFA to log in to the Organization, the returned value of `member_authenticated` will be `false`, and an `intermediate_session_token` will be returned.
        The `intermediate_session_token` can be passed into the [OTP SMS Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-otp-sms), [TOTP Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-totp),
        or [Recovery Codes Recover endpoint](https://stytch.com/docs/b2b/api/recovery-codes-recover) to complete the MFA step and acquire a full member session.
        The `intermediate_session_token` can also be used with the [Exchange Intermediate Session endpoint](https://stytch.com/docs/b2b/api/exchange-intermediate-session) or the [Create Organization via Discovery endpoint](https://stytch.com/docs/b2b/api/create-organization-via-discovery) to join a different Organization or create a new one.
        The `session_duration_minutes` and `session_custom_claims` parameters will be ignored.

        If a valid `session_token` or `session_jwt` is passed in, the Member will not be required to complete an MFA step.

        Fields:
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - email_address: The email address of the Member.
          - code: The code to authenticate.
          - session_token: A secret token for a given Stytch Session.
          - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
          - intermediate_session_token: The Intermediate Session Token. This token does not necessarily belong to a specific instance of a Member, but represents a bag of factors that may be converted to a member session. The token can be used with the [OTP SMS Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-otp-sms), [TOTP Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-totp), or [Recovery Codes Recover endpoint](https://stytch.com/docs/b2b/api/recovery-codes-recover) to complete an MFA flow and log in to the Organization. The token has a default expiry of 10 minutes. It can also be used with the [Exchange Intermediate Session endpoint](https://stytch.com/docs/b2b/api/exchange-intermediate-session) to join a specific Organization that allows the factors represented by the intermediate session token; or the [Create Organization via Discovery endpoint](https://stytch.com/docs/b2b/api/create-organization-via-discovery) to create a new Organization and Member. Intermediate Session Tokens have a default expiry of 10 minutes.
          - session_duration_minutes: Set the session lifetime to be this many minutes from now. This will start a new session if one doesn't already exist,
          returning both an opaque `session_token` and `session_jwt` for this session. Remember that the `session_jwt` will have a fixed lifetime of
          five minutes regardless of the underlying session duration, and will need to be refreshed over time.

          This value must be a minimum of 5 and a maximum of 527040 minutes (366 days).

          If a `session_token` or `session_jwt` is provided then a successful authentication will continue to extend the session this many minutes.

          If the `session_duration_minutes` parameter is not specified, a Stytch session will be created with a 60 minute duration. If you don't want
          to use the Stytch session product, you can ignore the session fields in the response.
          - session_custom_claims: Add a custom claims map to the Session being authenticated. Claims are only created if a Session is initialized by providing a value in
          `session_duration_minutes`. Claims will be included on the Session object and in the JWT. To update a key in an existing Session, supply a new value. To
          delete a key, supply a null value. Custom claims made with reserved claims (`iss`, `sub`, `aud`, `exp`, `nbf`, `iat`, `jti`) will be ignored.
          Total custom claims size cannot exceed four kilobytes.
          - locale: Used to determine which language to use when sending the user this delivery method. Parameter is a [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"`.

        Currently supported languages are English (`"en"`), Spanish (`"es"`), French (`"fr"`) and Brazilian Portuguese (`"pt-br"`); if no value is provided, the copy defaults to English.

        Request support for additional languages [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link")!

          - telemetry_id: If the `telemetry_id` is passed, as part of this request, Stytch will call the [Fingerprint Lookup API](https://stytch.com/docs/fraud/api/fingerprint-lookup) and store the associated fingerprints and IPGEO information for the Member. Your workspace must be enabled for Device Fingerprinting to use this feature.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "organization_id": organization_id,
            "email_address": email_address,
            "code": code,
        }
        if session_token is not None:
            data["session_token"] = session_token
        if session_jwt is not None:
            data["session_jwt"] = session_jwt
        if intermediate_session_token is not None:
            data["intermediate_session_token"] = intermediate_session_token
        if session_duration_minutes is not None:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            data["session_custom_claims"] = session_custom_claims
        if locale is not None:
            data["locale"] = locale
        if telemetry_id is not None:
            data["telemetry_id"] = telemetry_id

        url = self.api_base.url_for("/v1/b2b/otps/email/authenticate", data)
        res = await self.async_client.post(url, data, headers)
        return AuthenticateResponse.from_json(res.response.status, res.json)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Any, Dict, List, Optional

from stytch.b2b.models.idp_oauth import AuthorizeResponse, AuthorizeStartResponse
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class OAuth:
    def __init__(
        self, api_base: ApiBase, sync_client: SyncClient, async_client: AsyncClient
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    def authorize_start(
        self,
        client_id: str,
        redirect_uri: str,
        response_type: str,
        scopes: List[str],
        organization_id: Optional[str] = None,
        member_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        prompt: Optional[str] = None,
    ) -> AuthorizeStartResponse:
        """Initiates a request for authorization of a Connected App to access a Member's account.

        Call this endpoint using the query parameters from an OAuth Authorization request.
        This endpoint validates various fields (`scope`, `client_id`, `redirect_uri`, `prompt`, etc...) are correct and returns
        relevant information for rendering an OAuth Consent Screen.

        This endpoint returns:
        - A public representation of the Connected App requesting authorization
        - Whether _explicit_ consent must be granted before proceeding with the authorization
        - A list of scopes the Member has the ability to grant the Connected App

        Use this response to prompt the Member for consent (if necessary) before calling the [Submit OAuth Authorization](https://stytch.com/docs/b2b/api/connected-apps-oauth-authorize) endpoint.

        Exactly one of the following must be provided to identify the Member granting authorization:
        - `organization_id` + `member_id`
        - `session_token`
        - `session_jwt`

        If a `session_token` or `session_jwt` is passed, the OAuth Authorization will be linked to the Member's session for tracking purposes.
        One of these fields must be used if the Connected App intends to complete the [Exchange Access Token](https://stytch.com/docs/b2b/api/connected-app-access-token-exchange) flow.

        Fields:
          - client_id: The ID of the Connected App client.
          - redirect_uri: The callback URI used to redirect the user after authentication. This is the same URI provided at the start of the OAuth flow.  This field is required when using the `authorization_code` grant.
          - response_type: The OAuth 2.0 response type. For authorization code flows this value is `code`.
          - scopes: An array of scopes requested by the client.
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - member_id: Globally unique UUID that identifies a specific Member. The `member_id` is critical to perform operations on a Member, so be sure to preserve this value. You may use an external_id here if one is set for the member.
          - session_token: A secret token for a given Stytch Session.
          - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
          - prompt: Space separated list that specifies how the Authorization Server should prompt the user for reauthentication and consent. Only `consent` is supported today.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "client_id": client_id,
            "redirect_uri": redirect_uri,
            "response_type": response_type,
            "scopes": scopes,
        }
        if organization_id is not None:
            data["organization_id"] = organization_id
        if member_id is not None:
            data["member_id"] = member_id
        if session_token is not None:
            data["session_token"] = session_token
        if session_jwt is not None:
            data["session_jwt"] = session_jwt
        if prompt is not None:
            data["prompt"] = prompt

        url = self.api_base.url_for("/v1/b2b/idp/oauth/authorize/start", data)
        res = self.sync_client.post(url, data, headers)
        return AuthorizeStartResponse.from_json(res.response.status_code, res.json)

    async def authorize_start_async(
        self,
        client_id: str,
        redirect_uri: str,
        response_type: str,
        scopes: List[str],
        organization_id: Optional[str] = None,
        member_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        prompt: Optional[str] = None,
    ) -> AuthorizeStartResponse:
        """Initiates a request for authorization of a Connected App to access a Member's account.

        Call this endpoint using the query parameters from an OAuth Authorization request.
        This endpoint validates various fields (`scope`, `client_id`, `redirect_uri`, `prompt`, etc...) are correct and returns
        relevant information for rendering an OAuth Consent Screen.

        This endpoint returns:
        - A public representation of the Connected App requesting authorization
        - Whether _explicit_ consent must be granted before proceeding with the authorization
        - A list of scopes the Member has the ability to grant the Connected App

        Use this response to prompt the Member for consent (if necessary) before calling the [Submit OAuth Authorization](https://stytch.com/docs/b2b/api/connected-apps-oauth-authorize) endpoint.

        Exactly one of the following must be provided to identify the Member granting authorization:
        - `organization_id` + `member_id`
        - `session_token`
        - `session_jwt`

        If a `session_token` or `session_jwt` is passed, the OAuth Authorization will be linked to the Member's session for tracking purposes.
        One of these fields must be used if the Connected App intends to complete the [Exchange Access Token](https://stytch.com/docs/b2b/api/connected-app-access-token-exchange) flow.

        Fields:
          - client_id: The ID of the Connected App client.
          - redirect_uri: The callback URI used to redirect the user after authentication. This is the same URI provided at the start of the OAuth flow.  This field is required when using the `authorization_code` grant.
          - response_type: The OAuth 2.0 response type. For authorization code flows this value is `code`.
          - scopes: An array of scopes requested by the client.
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - member_id: Globally unique UUID that identifies a specific Member. The `member_id` is critical to perform operations on a Member, so be sure to preserve this value. You may use an external_id here if one is set for the member.
          - session_token: A secret token for a given Stytch Session.
          - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
          - prompt: Space separated list that specifies how the Authorization Server should prompt the user for reauthentication and consent. Only `consent` is supported today.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "client_id": client_id,
            "redirect_uri": redirect_uri,
            "response_type": response_type,
            "scopes": scopes,
        }
        if organization_id is not None:
            data["organization_id"] = organization_id
        if member_id is not None:
            data["member_id"] = member_id
        if session_token is not None:
            data["session_token"] = session_token
        if session_jwt is not None:
            data["session_jwt"] = session_jwt
        if prompt is not None:
            data["prompt"] = prompt

        url = self.api_base.url_for("/v1/b2b/idp/oauth/authorize/start", data)
        res = await self.async_client.post(url, data, headers)
        return AuthorizeStartResponse.from_json(res.response.status, res.json)

    def authorize(
        self,
        consent_granted: bool,
        scopes: List[str],
        client_id: str,
        redirect_uri: str,
        response_type: str,
        organization_id: Optional[str] = None,
        member_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        prompt: Optional[str] = None,
        state: Optional[str] = None,
        nonce: Optional[str] = None,
        code_challenge: Optional[str] = None,
        resources: Optional[List[str]] = None,
    ) -> AuthorizeResponse:
        """Completes a request for authorization of a Connected App to access a Member's account.

        Call this endpoint using the query parameters from an OAuth Authorization request, after previously validating those parameters using the
        [Preflight Check](https://stytch.com/docs/b2b/api/connected-apps-oauth-authorize-start) API.
        Note that this endpoint takes in a few additional parameters the preflight check does not- `state`, `nonce`, and `code_challenge`.

        If the authorization was successful, the `redirect_uri` will contain a valid `authorization_code` embedded as a query parameter.
        If the authorization was unsuccessful, the `redirect_uri` will contain an OAuth2.1 `error_code`.
        In both cases, redirect the Member to the location for the response to be consumed by the Connected App.

        Exactly one of the following must be provided to identify the Member granting authorization:
        - `organization_id` + `member_id`
        - `session_token`
        - `session_jwt`

        If a `session_token` or `session_jwt` is passed, the OAuth Authorization will be linked to the Member's session for tracking purposes.
        One of these fields must be used if the Connected App intends to complete the [Exchange Access Token](https://stytch.com/docs/b2b/api/connected-app-access-token-exchange) flow.

        Fields:
          - consent_granted: Indicates whether the user granted the requested scopes.
          - scopes: An array of scopes requested by the client.
          - client_id: The ID of the Connected App client.
          - redirect_uri: The callback URI used to redirect the user after authentication. This is the same URI provided at the start of the OAuth flow.  This field is required when using the `authorization_code` grant.
          - response_type: The OAuth 2.0 response type. For authorization code flows this value is `code`.
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - member_id: Globally unique UUID that identifies a specific Member. The `member_id` is critical to perform operations on a Member, so be sure to preserve this value. You may use an external_id here if one is set for the member.
          - session_token: A secret token for a given Stytch Session.
          - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
          - prompt: Space separated list that specifies how the Authorization Server should prompt the user for reauthentication and consent. Only `consent` is supported today.
          - state: An opaque value used to maintain state between the request and callback.
          - nonce: A string used to associate a client session with an ID token to mitigate replay attacks.
          - code_challenge: A base64url encoded challenge derived from the code verifier for PKCE flows.
          - resources: (no documentation yet)
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "consent_granted": consent_granted,
            "scopes": scopes,
            "client_id": client_id,
            "redirect_uri": redirect_uri,
            "response_type": response_type,
        }
        if organization_id is not None:
            data["organization_id"] = organization_id
        if member_id is not None:
            data["member_id"] = member_id
        if session_token is not None:
            data["session_token"] = session_token
        if session_jwt is not None:
            data["session_jwt"] = session_jwt
        if prompt is not None:
            data["prompt"] = prompt
        if state is not None:
            data["state"] = state
        if nonce is not None:
            data["nonce"] = nonce
        if code_challenge is not None:
            data["code_challenge"] = code_challenge
        if resources is not None:
            data["resources"] = resources

        url = self.api_base.url_for("/v1/b2b/idp/oauth/authorize", data)
        res = self.sync_client.post(url, data, headers)
        return AuthorizeResponse.from_json(res.response.status_code, res.json)

    async def authorize_async(
        self,
        consent_granted: bool,
        scopes: List[str],
        client_id: str,
        redirect_uri: str,
        response_type: str,
        organization_id: Optional[str] = None,
        member_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        prompt: Optional[str] = None,
        state: Optional[str] = None,
        nonce: Optional[str] = None,
        code_challenge: Optional[str] = None,
        resources: Optional[List[str]] = None,
    ) -> AuthorizeResponse:
        """Completes a request for authorization of a Connected App to access a Member's account.

        Call this endpoint using the query parameters from an OAuth Authorization request, after previously validating those parameters using the
        [Preflight Check](https://stytch.com/docs/b2b/api/connected-apps-oauth-authorize-start) API.
        Note that this endpoint takes in a few additional parameters the preflight check does not- `state`, `nonce`, and `code_challenge`.

        If the authorization was successful, the `redirect_uri` will contain a valid `authorization_code` embedded as a query parameter.
        If the authorization was unsuccessful, the `redirect_uri` will contain an OAuth2.1 `error_code`.
        In both cases, redirect the Member to the location for the response to be consumed by the Connected App.

        Exactly one of the following must be provided to identify the Member granting authorization:
        - `organization_id` + `member_id`
        - `session_token`
        - `session_jwt`

        If a `session_token` or `session_jwt` is passed, the OAuth Authorization will be linked to the Member's session for tracking purposes.
        One of these fields must be used if the Connected App intends to complete the [Exchange Access Token](https://stytch.com/docs/b2b/api/connected-app-access-token-exchange) flow.

        Fields:
          - consent_granted: Indicates whether the user granted the requested scopes.
          - scopes: An array of scopes requested by the client.
          - client_id: The ID of the Connected App client.
          - redirect_uri: The callback URI used to redirect the user after authentication. This is the same URI provided at the start of the OAuth flow.  This field is required when using the `authorization_code` grant.
          - response_type: The OAuth 2.0 response type. For authorization code flows this value is `code`.
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - member_id: Globally unique UUID that identifies a specific Member. The `member_id` is critical to perform operations on a Member, so be sure to preserve this value. You may use an external_id here if one is set for the member.
          - session_token: A secret token for a given Stytch Session.
          - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
          - prompt: Space separated list that specifies how the Authorization Server should prompt the user for reauthentication and consent. Only `consent` is supported today.
          - state: An opaque value used to maintain state between the request and callback.
          - nonce: A string used to associate a client session with an ID token to mitigate replay attacks.
          - code_challenge: A base64url encoded challenge derived from the code verifier for PKCE flows.
          - resources: (no documentation yet)
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "consent_granted": consent_granted,
            "scopes": scopes,
            "client_id": client_id,
            "redirect_uri": redirect_uri,
            "response_type": response_type,
        }
        if organization_id is not None:
            data["organization_id"] = organization_id
        if member_id is not None:
            data["member_id"] = member_id
        if session_token is not None:
            data["session_token"] = session_token
        if session_jwt is not None:
            data["session_jwt"] = session_jwt
        if prompt is not None:
            data["prompt"] = prompt
        if state is not None:
            data["state"] = state
        if nonce is not None:
            data["nonce"] = nonce
        if code_challenge is not None:
            data["code_challenge"] = code_challenge
        if resources is not None:
            data["resources"] = resources

        url = self.api_base.url_for("/v1/b2b/idp/oauth/authorize", data)
        res = await self.async_client.post(url, data, headers)
        return AuthorizeResponse.from_json(res.response.status, res.json)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

import datetime
import enum
from typing import Any, Dict, List, Optional

import pydantic

from stytch.b2b.models.mfa import MfaRequired
from stytch.b2b.models.organizations import Member, Organization
from stytch.consumer.models.device_history import DeviceInfo
from stytch.consumer.models.sessions import JWK, AuthenticationFactor
from stytch.core.response_base import ResponseBase
from stytch.shared.method_options import Authorization


class ExchangeRequestLocale(str, enum.Enum):
    EN = "en"
    ES = "es"
    PTBR = "pt-br"
    FR = "fr"
    IT = "it"
    DEDE = "de-DE"
    ZHHANS = "zh-Hans"
    CAES = "ca-ES"


class AuthorizationCheck(pydantic.BaseModel):
    """
    Fields:
      - organization_id: Globally unique UUID that identifies a specific Organization. The Organization's ID must match the Member's Organization
      - resource_id: A unique identifier of the RBAC Resource, provided by the developer and intended to be human-readable.

      A `resource_id` is not allowed to start with `stytch`, which is a special prefix used for Stytch default Resources with reserved `resource_id`s. These include:

      * `stytch.organization`
      * `stytch.member`
      * `stytch.sso`
      * `stytch.self`

      Check out the [guide on Stytch default Resources](https://stytch.com/docs/b2b/guides/rbac/stytch-default) for a more detailed explanation.


      - action: An action to take on a Resource.
    """  # noqa

    organization_id: str
    resource_id: str
    action: str


class AuthorizationVerdict(pydantic.BaseModel):
    """
    Fields:
      - authorized: Whether the Member was authorized to perform the specified action on the specified Resource. Always true if the request succeeds.
      - granting_roles: The complete list of Roles that gave the Member permission to perform the specified action on the specified Resource.
    """  # noqa

    authorized: bool
    granting_roles: List[str]


class MemberSession(pydantic.BaseModel):
    """
    Fields:
      - member_session_id: Globally unique UUID that identifies a specific Session.
      - member_id: Globally unique UUID that identifies a specific Member.
      - started_at: The timestamp when the Session was created. Values conform to the RFC 3339 standard and are expressed in UTC, e.g. `2021-12-29T12:33:09Z`.
      - last_accessed_at: The timestamp when the Session was last accessed. Values conform to the RFC 3339 standard and are expressed in UTC, e.g. `2021-12-29T12:33:09Z`.
      - expires_at: The timestamp when the Session expires. Values conform to the RFC 3339 standard and are expressed in UTC, e.g. `2021-12-29T12:33:09Z`.
      - authentication_factors: An array of different authentication factors that comprise a Session.
      - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value.
      - roles: (no documentation yet)
      - organization_slug: The unique URL slug of the Organization. The slug only accepts alphanumeric characters and the following reserved characters: `-` `.` `_` `~`. Must be between 2 and 128 characters in length. Wherever an organization_id is expected in a path or request parameter, you may also use the organization_slug as a convenience.
      - custom_claims: The custom claims map for a Session. Claims can be added to a session during a Sessions authenticate call.
    """  # noqa

    member_session_id: str
    member_id: str
    started_at: datetime.datetime
    last_accessed_at: datetime.datetime
    expires_at: datetime.datetime
    authentication_factors: List[AuthenticationFactor]
    organization_id: str
    roles: List[str]
    organization_slug: str
    custom_claims: Optional[Dict[str, Any]] = None


class PrimaryRequired(pydantic.BaseModel):
    """
    Fields:
      - allowed_auth_methods: Details the auth method that the member must also complete to fulfill the primary authentication requirements of the Organization. For example, a value of `[magic_link]` indicates that the Member must also complete a magic link authentication step. If you have an intermediate session token, you must pass it into that primary authentication step.
    """  # noqa

    allowed_auth_methods: List[str]


class RevokeRequestOptions(pydantic.BaseModel):
    """
    Fields:
      - authorization: Optional authorization object.
    Pass in an active Stytch Member session token or session JWT and the request
    will be run using that member's permissions.
    """  # noqa

    authorization: Optional[Authorization] = None

    def add_headers(self, headers: Dict[str, str]) -> Dict[str, str]:
        if self.authorization is not None:
            headers = self.authorization.add_headers(headers)
        return headers


class AttestResponse(ResponseBase):
    """Response type for `Sessions.attest`.
    Fields:
      - member_id: Globally unique UUID that identifies a specific Member.
      - member_session: The [Session object](https://stytch.com/docs/b2b/api/session-object).
      - session_token: A secret token for a given Stytch Session.
      - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
      - member: The [Member object](https://stytch.com/docs/b2b/api/member-object)
      - organization: The [Organization object](https://stytch.com/docs/b2b/api/organization-object).
      - member_device: If a valid `telemetry_id` was passed in the request and the [Fingerprint Lookup API](https://stytch.com/docs/fraud/api/fingerprint-lookup) returned results, the `member_device` response field will contain information about the member's device attributes.
    """  # noqa

    member_id: str
    member_session: MemberSession
    session_token: str
    session_jwt: str
    member: Member
    organization: Organization
    member_device: Optional[DeviceInfo] = None


class AuthenticateResponse(ResponseBase):
    """Response type for `Sessions.authenticate`.
    Fields:
      - member_session: The [Session object](https://stytch.com/docs/b2b/api/session-object).
      - session_token: A secret token for a given Stytch Session.
      - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
      - member: The [Member object](https://stytch.com/docs/b2b/api/member-object)
      - organization: The [Organization object](https://stytch.com/docs/b2b/api/organization-object).
      - verdict: If an `authorization_check` is provided in the request and the check succeeds, this field will return
      information about why the Member was granted permission.
    """  # noqa

    member_session: MemberSession
    session_token: str
    session_jwt: str
    member: Member
    organization: Organization
    verdict: Optional[AuthorizationVerdict] = None


class ExchangeAccessTokenResponse(ResponseBase):
    """Response type for `Sessions.exchange_access_token`.
    Fields:
      - member_id: Globally unique UUID that identifies a specific Member.
      - session_token: A secret token for a given Stytch Session.
      - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
      - member: The [Member object](https://stytch.com/docs/b2b/api/member-object)
      - organization: The [Organization object](https://stytch.com/docs/b2b/api/organization-object).
      - member_session: The [Session object](https://stytch.com/docs/b2b/api/session-object).
      - member_device: If a valid `telemetry_id` was passed in the request and the [Fingerprint Lookup API](https://stytch.com/docs/fraud/api/fingerprint-lookup) returned results, the `member_device` response field will contain information about the member's device attributes.
    """  # noqa

    member_id: str
    session_token: str
    session_jwt: str
    member: Member
    organization: Organization
    member_session: Optional[MemberSession] = None
    member_device: Optional[DeviceInfo] = None


class ExchangeResponse(ResponseBase):
    """Response type for `Sessions.exchange`.
    Fields:
      - member_id: Globally unique UUID that identifies a specific Member.
      - session_token: A secret token for a given Stytch Session.
      - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
      - member: The [Member object](https://stytch.com/docs/b2b/api/member-object)
      - organization: The [Organization object](https://stytch.com/docs/b2b/api/organization-object).
      - member_authenticated: Indicates whether the Member is fully authenticated. If false, the Member needs to complete an MFA step to log in to the Organization.
      - intermediate_session_token: The returned Intermediate Session Token contains any Email Magic Link or OAuth factors from the original member session that are valid for the target Organization. If this value is non-empty, the member must complete an MFA step to finish logging in to the Organization. The token can be used with the [OTP SMS Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-otp-sms), [TOTP Authenticate endpoint](https://stytch.com/docs/b2b/api/authenticate-totp), or [Recovery Codes Recover endpoint](https://stytch.com/docs/b2b/api/recovery-codes-recover) to complete an MFA flow and log in to the Organization. The token has a default expiry of 10 minutes. It can also be used with the [Exchange Intermediate Session endpoint](https://stytch.com/docs/b2b/api/exchange-intermediate-session) to join a specific Organization that allows the factors represented by the intermediate session token; or the [Create Organization via Discovery endpoint](https://stytch.com/docs/b2b/api/create-organization-via-discovery) to create a new Organization and Member. Intermediate Session Tokens have a default expiry of 10 minutes.
      - member_session: The [Session object](https://stytch.com/docs/b2b/api/session-object).
      - mfa_required: Information about the MFA requirements of the Organization and the Member's options for fulfilling MFA.
      - primary_required: Information about the primary authentication requirements of the Organization.
      - member_device: If a valid `telemetry_id` was passed in the request and the [Fingerprint Lookup API](https://stytch.com/docs/fraud/api/fingerprint-lookup) returned results, the `member_device` response field will contain information about the member's device attributes.
    """  # noqa

    member_id: str
    session_token: str
    session_jwt: str
    member: Member
    organization: Organization
    member_authenticated: bool
    intermediate_session_token: str
    member_session: Optional[MemberSession] = None
    mfa_required: Optional[MfaRequired] = None
    primary_required: Optional[PrimaryRequired] = None
    member_device: Optional[DeviceInfo] = None


class GetJWKSResponse(ResponseBase):
    """Response type for `Sessions.get_jwks`.
    Fields:
      - keys: The list of JWKs associated with the project.
    """  # noqa

    keys: List[JWK]


class GetResponse(ResponseBase):
    """Response type for `Sessions.get`.
    Fields:
      - member_sessions: An array of [Session objects](https://stytch.com/docs/b2b/api/session-object).
    """  # noqa

    member_sessions: List[MemberSession]


class MigrateResponse(ResponseBase):
    """Response type for `Sessions.migrate`.
    Fields:
      - member_id: Globally unique UUID that identifies a specific Member.
      - session_token: A secret token for a given Stytch Session.
      - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
      - member: The [Member object](https://stytch.com/docs/b2b/api/member-object)
      - organization: The [Organization object](https://stytch.com/docs/b2b/api/organization-object).
      - member_session: The [Session object](https://stytch.com/docs/b2b/api/session-object).
    """  # noqa

    member_id: str
    session_token: str
    session_jwt: str
    member: Member
    organization: Organization
    member_session: Optional[MemberSession] = None


class RevokeResponse(ResponseBase):
    """Response type for `Sessions.revoke`.
    Fields:
    """  # noqa


# MANUAL(LocalJWTResponse)(Types)
class LocalJWTResponse(pydantic.BaseModel):
    member_session: MemberSession
    roles_claim: Optional[List[str]]


# ENDMANUAL(LocalJWTResponse)


# MANUAL(AuthenticateJWTLocalResponse)(Types)
class AuthenticateJWTLocalResponse(ResponseBase):
    member_session: MemberSession
    session_jwt: str


# ENDMANUAL(AuthenticateJWTLocalResponse)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import List, Optional

import pydantic

from stytch.core.response_base import ResponseBase


class PolicyResource(pydantic.BaseModel):
    """
    Fields:
      - resource_id: A unique identifier of the RBAC Resource, provided by the developer and intended to be human-readable.

      A `resource_id` is not allowed to start with `stytch`, which is a special prefix used for Stytch default Resources with reserved `resource_id`s. These include:

      * `stytch.organization`
      * `stytch.member`
      * `stytch.sso`
      * `stytch.self`

      Check out the [guide on Stytch default Resources](https://stytch.com/docs/b2b/guides/rbac/stytch-default) for a more detailed explanation.


      - description: The description of the RBAC Resource.
      - actions: A list of all possible actions for a provided Resource.

      Reserved `actions` that are predefined by Stytch include:

      * `*`
      * For the `stytch.organization` Resource:
        * `update.info.name`
        * `update.info.slug`
        * `update.info.untrusted_metadata`
        * `update.info.email_jit_provisioning`
        * `update.info.logo_url`
        * `update.info.email_invites`
        * `update.info.allowed_domains`
        * `update.info.default_sso_connection`
        * `update.info.sso_jit_provisioning`
        * `update.info.mfa_policy`
        * `update.info.implicit_roles`
        * `delete`
      * For the `stytch.member` Resource:
        * `create`
        * `update.info.name`
        * `update.info.untrusted_metadata`
        * `update.info.mfa-phone`
        * `update.info.delete.mfa-phone`
        * `update.settings.is-breakglass`
        * `update.settings.mfa_enrolled`
        * `update.settings.roles`
        * `search`
        * `delete`
      * For the `stytch.sso` Resource:
        * `create`
        * `update`
        * `delete`
      * For the `stytch.self` Resource:
        * `update.info.name`
        * `update.info.untrusted_metadata`
        * `update.info.mfa-phone`
        * `update.info.delete.mfa-phone`
        * `update.info.delete.password`
        * `update.settings.mfa_enrolled`
        * `delete`

    """  # noqa

    resource_id: str
    description: str
    actions: List[str]


class PolicyRolePermission(pydantic.BaseModel):
    """
    Fields:
      - resource_id: A unique identifier of the RBAC Resource, provided by the developer and intended to be human-readable.

      A `resource_id` is not allowed to start with `stytch`, which is a special prefix used for Stytch default Resources with reserved `resource_id`s. These include:

      * `stytch.organization`
      * `stytch.member`
      * `stytch.sso`
      * `stytch.self`

      Check out the [guide on Stytch default Resources](https://stytch.com/docs/b2b/guides/rbac/stytch-default) for a more detailed explanation.


      - actions: A list of permitted actions the Scope is required to take with the provided Resource. You can use `*` as a wildcard to require a Scope permission to use all possible actions related to the Resource.
    """  # noqa

    resource_id: str
    actions: List[str]


class PolicyRole(pydantic.BaseModel):
    """
    Fields:
      - role_id: The unique identifier of the RBAC Role, provided by the developer and intended to be human-readable.

      Reserved `role_id`s that are predefined by Stytch include:

      * `stytch_member`
      * `stytch_admin`

      Check out the [guide on Stytch default Roles](https://stytch.com/docs/b2b/guides/rbac/stytch-default) for a more detailed explanation.


      - description: The description of the RBAC Role.
      - permissions: A list of permissions that link a [Resource](https://stytch.com/docs/b2b/api/rbac-resource-object) to a list of actions.
    """  # noqa

    role_id: str
    description: str
    permissions: List[PolicyRolePermission]


class OrgPolicy(pydantic.BaseModel):
    """
    Fields:
      - roles: An array of [Role objects](https://stytch.com/docs/b2b/api/rbac-role-object).
    """  # noqa

    roles: List[PolicyRole]


class PolicyScopePermission(pydantic.BaseModel):
    resource_id: str
    actions: List[str]


class PolicyScope(pydantic.BaseModel):
    scope: str
    description: str
    permissions: List[PolicyScopePermission]


class Policy(pydantic.BaseModel):
    """
    Fields:
      - roles: An array of [Role objects](https://stytch.com/docs/b2b/api/rbac-role-object).
      - resources: An array of [Resource objects](https://stytch.com/docs/b2b/api/rbac-resource-object).
      - scopes: (no documentation yet)
    """  # noqa

    roles: List[PolicyRole]
    resources: List[PolicyResource]
    scopes: List[PolicyScope]


class PolicyResponse(ResponseBase):
    """Response type for `RBAC.policy`.
    Fields:
      - policy: The RBAC Policy document that contains all defined Roles and Resources – which are managed in the [Dashboard](https://stytch.com/dashboard/rbac). Read more about these entities and how they work in our [RBAC overview](https://stytch.com/docs/b2b/guides/rbac/overview).
    """  # noqa

    policy: Optional[Policy] = None



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Any, Dict, Optional, Union

from stytch.b2b.models.passwords_session import ResetRequestLocale, ResetResponse
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class Sessions:
    def __init__(
        self, api_base: ApiBase, sync_client: SyncClient, async_client: AsyncClient
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    def reset(
        self,
        organization_id: str,
        password: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
        locale: Optional[Union[ResetRequestLocale, str]] = None,
        telemetry_id: Optional[str] = None,
    ) -> ResetResponse:
        """Reset the Member's password using their existing session. The endpoint will error if the session does not contain an authentication factor that has been issued within the last 5 minutes. Either `session_token` or `session_jwt` should be provided.

        Note that a successful password reset via an existing session will revoke all active sessions for the `member_id`, except for the one used during the reset flow.

        Fields:
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - password: The password to authenticate, reset, or set for the first time. Any UTF8 character is allowed, e.g. spaces, emojis, non-English characters, etc.
          - session_token: A secret token for a given Stytch Session.
          - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
          - session_duration_minutes: Set the session lifetime to be this many minutes from now. This will start a new session if one doesn't already exist,
          returning both an opaque `session_token` and `session_jwt` for this session. Remember that the `session_jwt` will have a fixed lifetime of
          five minutes regardless of the underlying session duration, and will need to be refreshed over time.

          This value must be a minimum of 5 and a maximum of 527040 minutes (366 days).

          If a `session_token` or `session_jwt` is provided then a successful authentication will continue to extend the session this many minutes.

          If the `session_duration_minutes` parameter is not specified, a Stytch session will be created with a 60 minute duration. If you don't want
          to use the Stytch session product, you can ignore the session fields in the response.
          - session_custom_claims: Add a custom claims map to the Session being authenticated. Claims are only created if a Session is initialized by providing a value in
          `session_duration_minutes`. Claims will be included on the Session object and in the JWT. To update a key in an existing Session, supply a new value. To
          delete a key, supply a null value. Custom claims made with reserved claims (`iss`, `sub`, `aud`, `exp`, `nbf`, `iat`, `jti`) will be ignored.
          Total custom claims size cannot exceed four kilobytes.
          - locale: Used to determine which language to use when sending the user this delivery method. Parameter is a [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"`.

        Currently supported languages are English (`"en"`), Spanish (`"es"`), French (`"fr"`) and Brazilian Portuguese (`"pt-br"`); if no value is provided, the copy defaults to English.

        Request support for additional languages [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link")!

          - telemetry_id: If the `telemetry_id` is passed, as part of this request, Stytch will call the [Fingerprint Lookup API](https://stytch.com/docs/fraud/api/fingerprint-lookup) and store the associated fingerprints and IPGEO information for the Member. Your workspace must be enabled for Device Fingerprinting to use this feature.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "organization_id": organization_id,
            "password": password,
        }
        if session_token is not None:
            data["session_token"] = session_token
        if session_jwt is not None:
            data["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            data["session_custom_claims"] = session_custom_claims
        if locale is not None:
            data["locale"] = locale
        if telemetry_id is not None:
            data["telemetry_id"] = telemetry_id

        url = self.api_base.url_for("/v1/b2b/passwords/session/reset", data)
        res = self.sync_client.post(url, data, headers)
        return ResetResponse.from_json(res.response.status_code, res.json)

    async def reset_async(
        self,
        organization_id: str,
        password: str,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
        session_duration_minutes: Optional[int] = None,
        session_custom_claims: Optional[Dict[str, Any]] = None,
        locale: Optional[ResetRequestLocale] = None,
        telemetry_id: Optional[str] = None,
    ) -> ResetResponse:
        """Reset the Member's password using their existing session. The endpoint will error if the session does not contain an authentication factor that has been issued within the last 5 minutes. Either `session_token` or `session_jwt` should be provided.

        Note that a successful password reset via an existing session will revoke all active sessions for the `member_id`, except for the one used during the reset flow.

        Fields:
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - password: The password to authenticate, reset, or set for the first time. Any UTF8 character is allowed, e.g. spaces, emojis, non-English characters, etc.
          - session_token: A secret token for a given Stytch Session.
          - session_jwt: The JSON Web Token (JWT) for a given Stytch Session.
          - session_duration_minutes: Set the session lifetime to be this many minutes from now. This will start a new session if one doesn't already exist,
          returning both an opaque `session_token` and `session_jwt` for this session. Remember that the `session_jwt` will have a fixed lifetime of
          five minutes regardless of the underlying session duration, and will need to be refreshed over time.

          This value must be a minimum of 5 and a maximum of 527040 minutes (366 days).

          If a `session_token` or `session_jwt` is provided then a successful authentication will continue to extend the session this many minutes.

          If the `session_duration_minutes` parameter is not specified, a Stytch session will be created with a 60 minute duration. If you don't want
          to use the Stytch session product, you can ignore the session fields in the response.
          - session_custom_claims: Add a custom claims map to the Session being authenticated. Claims are only created if a Session is initialized by providing a value in
          `session_duration_minutes`. Claims will be included on the Session object and in the JWT. To update a key in an existing Session, supply a new value. To
          delete a key, supply a null value. Custom claims made with reserved claims (`iss`, `sub`, `aud`, `exp`, `nbf`, `iat`, `jti`) will be ignored.
          Total custom claims size cannot exceed four kilobytes.
          - locale: Used to determine which language to use when sending the user this delivery method. Parameter is a [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"`.

        Currently supported languages are English (`"en"`), Spanish (`"es"`), French (`"fr"`) and Brazilian Portuguese (`"pt-br"`); if no value is provided, the copy defaults to English.

        Request support for additional languages [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link")!

          - telemetry_id: If the `telemetry_id` is passed, as part of this request, Stytch will call the [Fingerprint Lookup API](https://stytch.com/docs/fraud/api/fingerprint-lookup) and store the associated fingerprints and IPGEO information for the Member. Your workspace must be enabled for Device Fingerprinting to use this feature.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "organization_id": organization_id,
            "password": password,
        }
        if session_token is not None:
            data["session_token"] = session_token
        if session_jwt is not None:
            data["session_jwt"] = session_jwt
        if session_duration_minutes is not None:
            data["session_duration_minutes"] = session_duration_minutes
        if session_custom_claims is not None:
            data["session_custom_claims"] = session_custom_claims
        if locale is not None:
            data["locale"] = locale
        if telemetry_id is not None:
            data["telemetry_id"] = telemetry_id

        url = self.api_base.url_for("/v1/b2b/passwords/session/reset", data)
        res = await self.async_client.post(url, data, headers)
        return ResetResponse.from_json(res.response.status, res.json)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

import pydantic


class ScopeResult(pydantic.BaseModel):
    """
    Fields:
      - scope: The name of the scope.
      - description: A human-readable description of the scope, taken from the RBAC Policy.
      - is_grantable: Indicates whether the scope can be granted. Users can only grant scopes if they have the required permissions.
    """  # noqa

    scope: str
    description: str
    is_grantable: bool



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import List, Optional

import pydantic

from stytch.core.response_base import ResponseBase


class PolicyResource(pydantic.BaseModel):
    """
    Fields:
      - resource_id: A unique identifier of the RBAC Resource, provided by the developer and intended to be human-readable.

      A `resource_id` is not allowed to start with `stytch`, which is a special prefix used for Stytch default Resources with reserved `resource_id`s.

      - description: The description of the RBAC Resource.
      - actions: A list of all possible actions for a provided Resource.
    """  # noqa

    resource_id: str
    description: str
    actions: List[str]


class PolicyRolePermission(pydantic.BaseModel):
    """
    Fields:
      - resource_id: A unique identifier of the RBAC Resource, provided by the developer and intended to be human-readable.

      A `resource_id` is not allowed to start with `stytch`, which is a special prefix used for Stytch default Resources with reserved `resource_id`s.

      - actions: A list of permitted actions the Role is authorized to take with the provided Resource. You can use `*` as a wildcard to grant a Role permission to use all possible actions related to the Resource.
    """  # noqa

    resource_id: str
    actions: List[str]


class PolicyRole(pydantic.BaseModel):
    """
    Fields:
      - role_id: The unique identifier of the RBAC Role, provided by the developer and intended to be human-readable.

      The `stytch_user` `role_id` is predefined by Stytch.
      Check out the [RBAC guide](https://stytch.com/docs/guides/rbac/overview) for a more detailed explanation.

      - description: The description of the RBAC Role.
      - permissions: A list of permissions that link a [Resource](https://stytch.com/docs/api/rbac-resource-object) to a list of actions.
    """  # noqa

    role_id: str
    description: str
    permissions: List[PolicyRolePermission]


class PolicyScopePermission(pydantic.BaseModel):
    resource_id: str
    actions: List[str]


class PolicyScope(pydantic.BaseModel):
    scope: str
    description: str
    permissions: List[PolicyScopePermission]


class Policy(pydantic.BaseModel):
    """
    Fields:
      - roles: An array of [Role objects](https://stytch.com/docs/api/rbac-role-object).
      - resources: An array of [Resource objects](https://stytch.com/docs/api/rbac-resource-object).
      - scopes: (no documentation yet)
    """  # noqa

    roles: List[PolicyRole]
    resources: List[PolicyResource]
    scopes: List[PolicyScope]


class PolicyResponse(ResponseBase):
    """Response type for `RBAC.policy`.
    Fields:
      - policy: The RBAC Policy document that contains all defined Roles and Resources – which are managed in the [Dashboard](https://stytch.com/dashboard/rbac). Read more about these entities and how they work in our [RBAC overview](https://stytch.com/docs/guides/rbac/overview).
    """  # noqa

    policy: Optional[Policy] = None



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Any, Dict, List, Optional

import pydantic

from stytch.core.response_base import ResponseBase


class ScopeResult(pydantic.BaseModel):
    """
    Fields:
      - scope: The name of the scope.
      - description: A human-readable description of the scope, taken from the RBAC Policy.
      - is_grantable: Indicates whether the scope can be granted. Users can only grant scopes if they have the required permissions.
    """  # noqa

    scope: str
    description: str
    is_grantable: bool


# MANUAL(IDP)(TYPES)
# ADDIMPORT: from stytch.core.response_base import ResponseBase


class IDPTokenResponse(ResponseBase):
    """Response type for `IDP.introspect_token_network`.
    Fields:
      - active: Whether or not this token is active.
      - sub: Subject of this token.
      - scope: A space-delimited string of scopes this token is granted.
      - aud: Audience of this token. Usually the user or member ID, and any custom audience, if present.
      - exp: Expiration of this access token, in Unix time.
      - iat: The time this access token was issued.
      - iss: The issuer of this access token.
      - nbf: The time before which the token must not be accepted for processing.
      - token_type: The type of token. Possible values are `access_token` and `refresh_token`.
    """  # noqa

    active: bool
    sub: Optional[str] = None
    scope: Optional[str] = None
    aud: Optional[List[str]] = []
    exp: Optional[int] = None
    iat: Optional[int] = None
    iss: Optional[str] = None
    nbf: Optional[int] = None
    token_type: Optional[str] = None


class IDPTokenClaims(pydantic.BaseModel):
    """Response type for `IDP.introspect_token_network`.
    Fields:
      - subject: The subject (either user_id or member_id) that the token is intended for.
      - scope: A space-delimited string of scopes this token is granted.
      - custom_claims: A dict of custom claims of the token.
      - audience: Audience of this token. Usually the user or member ID, and any custom audience, if present.
      - expires_at: Expiration of this access token, in Unix time.
      - issued_at: The time this access token was issued.
      - issuer: The issuer of this access token.
      - not_before: The time before which the token must not be accepted for processing.
      - token_type: The type of this token - e.g. 'access_token' or 'refresh_token'.
    """  # noqa

    subject: str
    scope: Optional[str]
    custom_claims: Optional[Dict[str, Any]] = None
    audience: Optional[List[str]]
    expires_at: Optional[int]
    issued_at: Optional[int]
    issuer: Optional[str]
    not_before: Optional[int]
    token_type: Optional[str]
    organization_claim: Optional[Dict[str, Any]] = None


# ENDMANUAL(IDP)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

import enum

from stytch.consumer.models.fraud import AddressInformation, DomainInformation
from stytch.core.response_base import ResponseBase


class RiskResponseAction(str, enum.Enum):
    ALLOW = "ALLOW"
    CHALLENGE = "CHALLENGE"
    BLOCK = "BLOCK"


class RiskResponse(ResponseBase):
    """Response type for `Email.risk`.
    Fields:
      - address_information: Information about the email address.
      - domain_information: Information about the email domain.
      - action: The suggested action based on the attributes of the email address. The available actions are:
      * `ALLOW` - This email is most likely safe to send to and not fraudulent.
      * `BLOCK` - This email is invalid or exhibits signs of fraud. We recommend blocking the end user.
      * `CHALLENGE` - This email has some potentially fraudulent attributes. We recommend increased friction such as 2FA or other forms of extended user verification before allowing the privileged action to proceed.

      - risk_score: A score from 0 to 100 indicating how risky the email is. 100 is the most risky.
    """  # noqa

    address_information: AddressInformation
    domain_information: DomainInformation
    action: RiskResponseAction
    risk_score: int



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from stytch.consumer.models.connected_apps import (
    ConnectedApp,
    ConnectedAppWithNextClientSecret,
)
from stytch.core.response_base import ResponseBase


class RotateCancelResponse(ResponseBase):
    """Response type for `Secrets.rotate_cancel`.
    Fields:
      - connected_app: The Connected App affected by this operation.
    """  # noqa

    connected_app: ConnectedApp


class RotateResponse(ResponseBase):
    """Response type for `Secrets.rotate`.
    Fields:
      - connected_app: The Connected App affected by this operation.
    """  # noqa

    connected_app: ConnectedApp


class RotateStartResponse(ResponseBase):
    """Response type for `Secrets.rotate_start`.
    Fields:
      - connected_app: The Connected App affected by this operation.
    """  # noqa

    connected_app: ConnectedAppWithNextClientSecret



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Any, Dict, Set, Union

from stytch.b2b.models.rbac import (
    OrgPolicy,
    Policy as B2BPolicy,
    PolicyResource,
)
from stytch.b2b.models.rbac_organizations import (
    GetOrgPolicyResponse,
    SetOrgPolicyResponse,
)
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class Organizations:
    def __init__(
        self, api_base: ApiBase, sync_client: SyncClient, async_client: AsyncClient
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    def get_org_policy(
        self,
        organization_id: str,
    ) -> GetOrgPolicyResponse:
        """Get the active RBAC Policy for a specific Organization within your Stytch Project. An Organization RBAC Policy contains the roles that have been defined specifically for that organization, allowing for organization-specific permissioning models.

        This endpoint returns the organization-scoped roles that supplement the project-level RBAC policy. Organization policies allow you to define custom roles that are specific to individual organizations within your project.

        When using the backend SDKs, the RBAC Policy will be cached to allow for local evaluations, eliminating the need for an extra request to Stytch. The policy will be refreshed if an authorization check is requested and the RBAC policy was last updated more than 5 minutes ago.

        Organization-specific roles can be created and managed through this API endpoint, providing fine-grained control over permissions at the organization level.

        Check out the [RBAC overview](https://stytch.com/docs/b2b/guides/rbac/overview) to learn more about Stytch's RBAC permissioning model and organization-scoped policies.

        Fields:
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "organization_id": organization_id,
        }

        url = self.api_base.url_for(
            "/v1/b2b/rbac/organizations/{organization_id}", data
        )
        res = self.sync_client.get(url, data, headers)
        return GetOrgPolicyResponse.from_json(res.response.status_code, res.json)

    async def get_org_policy_async(
        self,
        organization_id: str,
    ) -> GetOrgPolicyResponse:
        """Get the active RBAC Policy for a specific Organization within your Stytch Project. An Organization RBAC Policy contains the roles that have been defined specifically for that organization, allowing for organization-specific permissioning models.

        This endpoint returns the organization-scoped roles that supplement the project-level RBAC policy. Organization policies allow you to define custom roles that are specific to individual organizations within your project.

        When using the backend SDKs, the RBAC Policy will be cached to allow for local evaluations, eliminating the need for an extra request to Stytch. The policy will be refreshed if an authorization check is requested and the RBAC policy was last updated more than 5 minutes ago.

        Organization-specific roles can be created and managed through this API endpoint, providing fine-grained control over permissions at the organization level.

        Check out the [RBAC overview](https://stytch.com/docs/b2b/guides/rbac/overview) to learn more about Stytch's RBAC permissioning model and organization-scoped policies.

        Fields:
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "organization_id": organization_id,
        }

        url = self.api_base.url_for(
            "/v1/b2b/rbac/organizations/{organization_id}", data
        )
        res = await self.async_client.get(url, data, headers)
        return GetOrgPolicyResponse.from_json(res.response.status, res.json)

    def set_org_policy(
        self,
        organization_id: str,
        org_policy: Union[OrgPolicy, Dict[str, Any]],
    ) -> SetOrgPolicyResponse:
        """Set the RBAC Policy for a specific Organization within your Stytch Project. An Organization RBAC Policy allows you to define roles that are specific to that organization, providing fine-grained control over permissions at the organization level.

        This endpoint allows you to create, update, or replace the organization-scoped roles for a given organization. Organization policies supplement the project-level RBAC policy with additional roles that are only applicable within the context of that specific organization.

        The organization policy consists of roles, where each role defines:
        - A unique `role_id` to identify the role
        - A human-readable `description` of the role's purpose
        - A set of `permissions` that specify which actions can be performed on which resources

        When you set an organization policy, it will replace any existing organization-specific roles for that organization. The project-level RBAC policy remains unchanged.

        Organization-specific roles are useful for scenarios where different organizations within your project require different permission structures, such as:
        - Multi-tenant applications with varying access levels per tenant
        - Organizations with custom approval workflows
        - Different organizational hierarchies requiring unique role definitions

        Check out the [RBAC overview](https://stytch.com/docs/b2b/guides/rbac/overview) to learn more about Stytch's RBAC permissioning model and organization-scoped policies.

        Fields:
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - org_policy: The organization-specific RBAC Policy that contains roles defined for this organization. Organization policies supplement the project-level RBAC policy with additional roles that are specific to the organization.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "organization_id": organization_id,
            "org_policy": (
                org_policy if isinstance(org_policy, dict) else org_policy.dict()
            ),
        }

        url = self.api_base.url_for(
            "/v1/b2b/rbac/organizations/{organization_id}", data
        )
        res = self.sync_client.put(url, data, headers)
        return SetOrgPolicyResponse.from_json(res.response.status_code, res.json)

    async def set_org_policy_async(
        self,
        organization_id: str,
        org_policy: OrgPolicy,
    ) -> SetOrgPolicyResponse:
        """Set the RBAC Policy for a specific Organization within your Stytch Project. An Organization RBAC Policy allows you to define roles that are specific to that organization, providing fine-grained control over permissions at the organization level.

        This endpoint allows you to create, update, or replace the organization-scoped roles for a given organization. Organization policies supplement the project-level RBAC policy with additional roles that are only applicable within the context of that specific organization.

        The organization policy consists of roles, where each role defines:
        - A unique `role_id` to identify the role
        - A human-readable `description` of the role's purpose
        - A set of `permissions` that specify which actions can be performed on which resources

        When you set an organization policy, it will replace any existing organization-specific roles for that organization. The project-level RBAC policy remains unchanged.

        Organization-specific roles are useful for scenarios where different organizations within your project require different permission structures, such as:
        - Multi-tenant applications with varying access levels per tenant
        - Organizations with custom approval workflows
        - Different organizational hierarchies requiring unique role definitions

        Check out the [RBAC overview](https://stytch.com/docs/b2b/guides/rbac/overview) to learn more about Stytch's RBAC permissioning model and organization-scoped policies.

        Fields:
          - organization_id: Globally unique UUID that identifies a specific Organization. The `organization_id` is critical to perform operations on an Organization, so be sure to preserve this value. You may also use the organization_slug or organization_external_id here as a convenience.
          - org_policy: The organization-specific RBAC Policy that contains roles defined for this organization. Organization policies supplement the project-level RBAC policy with additional roles that are specific to the organization.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "organization_id": organization_id,
            "org_policy": (
                org_policy if isinstance(org_policy, dict) else org_policy.dict()
            ),
        }

        url = self.api_base.url_for(
            "/v1/b2b/rbac/organizations/{organization_id}", data
        )
        res = await self.async_client.put(url, data, headers)
        return SetOrgPolicyResponse.from_json(res.response.status, res.json)

    # MANUAL(validate_org_policy)(SERVICE_METHOD)
    # ADDIMPORT: from stytch.b2b.models.rbac import Policy as B2BPolicy
    # ADDIMPORT: from typing import Set
    @staticmethod
    def validate_org_policy(project_policy: B2BPolicy, org_policy: OrgPolicy) -> None:
        project_roles = set({r.role_id for r in project_policy.roles})
        project_resources = {r.resource_id: r for r in project_policy.resources}

        org_roles: Set[str] = set()
        for role in org_policy.roles:
            org_role_id = role.role_id
            if org_role_id in org_roles:
                raise Exception(
                    f"Duplicate role {org_role_id} in Organization RBAC policy"
                )
            org_roles.add(org_role_id)

            if org_role_id in project_roles:
                raise Exception(
                    f"Role {org_role_id} already defined in Project RBAC policy"
                )

            for permission in role.permissions:
                resource_id = permission.resource_id
                if not resource_id in project_resources:
                    raise Exception(
                        f"Resource {resource_id} not defined in Project RBAC policy"
                    )

                if len(permission.actions) == 0:
                    raise Exception(
                        f"No actions defined for role {org_role_id}, resource {resource_id}"
                    )
                if len(permission.actions) == 1 and "*" == permission.actions[0]:
                    continue
                if len(permission.actions) > 1 and "*" in permission.actions:
                    raise Exception(
                        "Wildcard actions must be the only action defined for a role and resource"
                    )

                project_resource = project_resources[resource_id]
                for action in permission.actions:
                    if action.strip() == "":
                        raise Exception(
                            f"Empty action on resource {resource_id} is not permitted"
                        )

                    if not action in project_resource.actions:
                        raise Exception(
                            f"Unknown action {action} defined on resource {resource_id}"
                        )

        return

    # ENDMANUAL(validate_org_policy)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Any, Dict

from stytch.consumer.models.fraud_email import RiskResponse
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class Email:
    def __init__(
        self, api_base: ApiBase, sync_client: SyncClient, async_client: AsyncClient
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    def risk(
        self,
        email_address: str,
    ) -> RiskResponse:
        """Get risk information for a specific email address.
        The response will contain a recommended action (`ALLOW`, `BLOCK`, or `CHALLENGE`) and a more granular `risk_score`.
        You can also check the `address_information` and `domain_information` fields for more information about the email address and email domain.

        This feature is in beta. Reach out to us [here](mailto:fraud-team@stytch.com?subject=Email_Intelligence_Early_Access) if you'd like to request early access.

        Fields:
          - email_address: The email address to check.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "email_address": email_address,
        }

        url = self.api_base.url_for("/v1/email/risk", data)
        res = self.sync_client.post(url, data, headers)
        return RiskResponse.from_json(res.response.status_code, res.json)

    async def risk_async(
        self,
        email_address: str,
    ) -> RiskResponse:
        """Get risk information for a specific email address.
        The response will contain a recommended action (`ALLOW`, `BLOCK`, or `CHALLENGE`) and a more granular `risk_score`.
        You can also check the `address_information` and `domain_information` fields for more information about the email address and email domain.

        This feature is in beta. Reach out to us [here](mailto:fraud-team@stytch.com?subject=Email_Intelligence_Early_Access) if you'd like to request early access.

        Fields:
          - email_address: The email address to check.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "email_address": email_address,
        }

        url = self.api_base.url_for("/v1/email/risk", data)
        res = await self.async_client.post(url, data, headers)
        return RiskResponse.from_json(res.response.status, res.json)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Any, Dict, Optional, Union

from stytch.consumer.models.attribute import Attributes
from stytch.consumer.models.otp_whatsapp import (
    LoginOrCreateRequestLocale,
    LoginOrCreateResponse,
    SendRequestLocale,
    SendResponse,
)
from stytch.core.api_base import ApiBase
from stytch.core.http.client import AsyncClient, SyncClient


class WhatsApp:
    def __init__(
        self, api_base: ApiBase, sync_client: SyncClient, async_client: AsyncClient
    ) -> None:
        self.api_base = api_base
        self.sync_client = sync_client
        self.async_client = async_client

    def send(
        self,
        phone_number: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Union[Attributes, Dict[str, Any]]] = None,
        locale: Optional[Union[SendRequestLocale, str]] = None,
        user_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
    ) -> SendResponse:
        """Send a One-Time Passcode (OTP) to a User's WhatsApp. If you'd like to create a user and send them a passcode with one request, use our [log in or create](https://stytch.com/docs/api/whatsapp-login-or-create) endpoint.

        Note that sending another OTP code before the first has expired will invalidate the first code.

        ### Cost to send SMS OTP
        Before configuring SMS or WhatsApp OTPs, please review how Stytch [bills the costs of international OTPs](https://stytch.com/pricing) and understand how to protect your app against [toll fraud](https://stytch.com/docs/guides/passcodes/toll-fraud/overview).

        ### Add a phone number to an existing user

        This endpoint also allows you to add a new phone number to an existing Stytch User. Including a `user_id`, `session_token`, or `session_jwt` in your Send one-time passcode by WhatsApp request will add the new, unverified phone number to the existing Stytch User. If the user successfully authenticates within 5 minutes, the new phone number will be marked as verified and remain permanently on the existing Stytch User. Otherwise, it will be removed from the User object, and any subsequent login requests using that phone number will create a new User.

        ### Next steps

        Collect the OTP which was delivered to the user. Call [Authenticate OTP](https://stytch.com/docs/api/authenticate-otp) using the OTP `code` along with the `phone_id` found in the response as the `method_id`.

        Fields:
          - phone_number: The phone number to use for one-time passcodes. The phone number should be in E.164 format (i.e. +1XXXXXXXXXX). You may use +10000000000 to test this endpoint, see [Testing](https://stytch.com/docs/home#resources_testing) for more detail.
          - expiration_minutes: Set the expiration for the one-time passcode, in minutes. The minimum expiration is 1 minute and the maximum is 10 minutes. The default expiration is 2 minutes.
          - attributes: Provided attributes to help with fraud detection. These values are pulled and passed into Stytch endpoints by your application.
          - locale: Used to determine which language to use when sending the user this delivery method. Parameter is a [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"`.

        Currently supported languages are English (`"en"`), Spanish (`"es"`), French (`"fr"`) and Brazilian Portuguese (`"pt-br"`); if no value is provided, the copy defaults to English.

        Request support for additional languages [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link")!

          - user_id: The unique ID of a specific User. You may use an `external_id` here if one is set for the user.
          - session_token: The `session_token` associated with a User's existing Session.
          - session_jwt: The `session_jwt` associated with a User's existing Session.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "phone_number": phone_number,
        }
        if expiration_minutes is not None:
            data["expiration_minutes"] = expiration_minutes
        if attributes is not None:
            data["attributes"] = (
                attributes if isinstance(attributes, dict) else attributes.dict()
            )
        if locale is not None:
            data["locale"] = locale
        if user_id is not None:
            data["user_id"] = user_id
        if session_token is not None:
            data["session_token"] = session_token
        if session_jwt is not None:
            data["session_jwt"] = session_jwt

        url = self.api_base.url_for("/v1/otps/whatsapp/send", data)
        res = self.sync_client.post(url, data, headers)
        return SendResponse.from_json(res.response.status_code, res.json)

    async def send_async(
        self,
        phone_number: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Attributes] = None,
        locale: Optional[SendRequestLocale] = None,
        user_id: Optional[str] = None,
        session_token: Optional[str] = None,
        session_jwt: Optional[str] = None,
    ) -> SendResponse:
        """Send a One-Time Passcode (OTP) to a User's WhatsApp. If you'd like to create a user and send them a passcode with one request, use our [log in or create](https://stytch.com/docs/api/whatsapp-login-or-create) endpoint.

        Note that sending another OTP code before the first has expired will invalidate the first code.

        ### Cost to send SMS OTP
        Before configuring SMS or WhatsApp OTPs, please review how Stytch [bills the costs of international OTPs](https://stytch.com/pricing) and understand how to protect your app against [toll fraud](https://stytch.com/docs/guides/passcodes/toll-fraud/overview).

        ### Add a phone number to an existing user

        This endpoint also allows you to add a new phone number to an existing Stytch User. Including a `user_id`, `session_token`, or `session_jwt` in your Send one-time passcode by WhatsApp request will add the new, unverified phone number to the existing Stytch User. If the user successfully authenticates within 5 minutes, the new phone number will be marked as verified and remain permanently on the existing Stytch User. Otherwise, it will be removed from the User object, and any subsequent login requests using that phone number will create a new User.

        ### Next steps

        Collect the OTP which was delivered to the user. Call [Authenticate OTP](https://stytch.com/docs/api/authenticate-otp) using the OTP `code` along with the `phone_id` found in the response as the `method_id`.

        Fields:
          - phone_number: The phone number to use for one-time passcodes. The phone number should be in E.164 format (i.e. +1XXXXXXXXXX). You may use +10000000000 to test this endpoint, see [Testing](https://stytch.com/docs/home#resources_testing) for more detail.
          - expiration_minutes: Set the expiration for the one-time passcode, in minutes. The minimum expiration is 1 minute and the maximum is 10 minutes. The default expiration is 2 minutes.
          - attributes: Provided attributes to help with fraud detection. These values are pulled and passed into Stytch endpoints by your application.
          - locale: Used to determine which language to use when sending the user this delivery method. Parameter is a [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"`.

        Currently supported languages are English (`"en"`), Spanish (`"es"`), French (`"fr"`) and Brazilian Portuguese (`"pt-br"`); if no value is provided, the copy defaults to English.

        Request support for additional languages [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link")!

          - user_id: The unique ID of a specific User. You may use an `external_id` here if one is set for the user.
          - session_token: The `session_token` associated with a User's existing Session.
          - session_jwt: The `session_jwt` associated with a User's existing Session.
        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "phone_number": phone_number,
        }
        if expiration_minutes is not None:
            data["expiration_minutes"] = expiration_minutes
        if attributes is not None:
            data["attributes"] = (
                attributes if isinstance(attributes, dict) else attributes.dict()
            )
        if locale is not None:
            data["locale"] = locale
        if user_id is not None:
            data["user_id"] = user_id
        if session_token is not None:
            data["session_token"] = session_token
        if session_jwt is not None:
            data["session_jwt"] = session_jwt

        url = self.api_base.url_for("/v1/otps/whatsapp/send", data)
        res = await self.async_client.post(url, data, headers)
        return SendResponse.from_json(res.response.status, res.json)

    def login_or_create(
        self,
        phone_number: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Union[Attributes, Dict[str, Any]]] = None,
        create_user_as_pending: Optional[bool] = None,
        locale: Optional[Union[LoginOrCreateRequestLocale, str]] = None,
    ) -> LoginOrCreateResponse:
        """Send a one-time passcode (OTP) to a User's WhatsApp using their phone number. If the phone number is not associated with a User already, a User will be created.

        ### Cost to send SMS OTP
        Before configuring SMS or WhatsApp OTPs, please review how Stytch [bills the costs of international OTPs](https://stytch.com/pricing) and understand how to protect your app against [toll fraud](https://stytch.com/docs/guides/passcodes/toll-fraud/overview).

        ### Next steps

        Collect the OTP which was delivered to the User. Call [Authenticate OTP](https://stytch.com/docs/api/authenticate-otp) using the OTP `code` along with the `phone_id` found in the response as the `method_id`.

        Fields:
          - phone_number: The phone number to use for one-time passcodes. The phone number should be in E.164 format (i.e. +1XXXXXXXXXX). You may use +10000000000 to test this endpoint, see [Testing](https://stytch.com/docs/home#resources_testing) for more detail.
          - expiration_minutes: Set the expiration for the one-time passcode, in minutes. The minimum expiration is 1 minute and the maximum is 10 minutes. The default expiration is 2 minutes.
          - attributes: Provided attributes to help with fraud detection. These values are pulled and passed into Stytch endpoints by your application.
          - create_user_as_pending: Flag for whether or not to save a user as pending vs active in Stytch. Defaults to false.
                If true, users will be saved with status pending in Stytch's backend until authenticated.
                If false, users will be created as active. An example usage of
                a true flag would be to require users to verify their phone by entering the OTP code before creating
                an account for them.
          - locale: Used to determine which language to use when sending the user this delivery method. Parameter is a [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"`.

        Currently supported languages are English (`"en"`), Spanish (`"es"`), French (`"fr"`) and Brazilian Portuguese (`"pt-br"`); if no value is provided, the copy defaults to English.

        Request support for additional languages [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link")!

        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "phone_number": phone_number,
        }
        if expiration_minutes is not None:
            data["expiration_minutes"] = expiration_minutes
        if attributes is not None:
            data["attributes"] = (
                attributes if isinstance(attributes, dict) else attributes.dict()
            )
        if create_user_as_pending is not None:
            data["create_user_as_pending"] = create_user_as_pending
        if locale is not None:
            data["locale"] = locale

        url = self.api_base.url_for("/v1/otps/whatsapp/login_or_create", data)
        res = self.sync_client.post(url, data, headers)
        return LoginOrCreateResponse.from_json(res.response.status_code, res.json)

    async def login_or_create_async(
        self,
        phone_number: str,
        expiration_minutes: Optional[int] = None,
        attributes: Optional[Attributes] = None,
        create_user_as_pending: Optional[bool] = None,
        locale: Optional[LoginOrCreateRequestLocale] = None,
    ) -> LoginOrCreateResponse:
        """Send a one-time passcode (OTP) to a User's WhatsApp using their phone number. If the phone number is not associated with a User already, a User will be created.

        ### Cost to send SMS OTP
        Before configuring SMS or WhatsApp OTPs, please review how Stytch [bills the costs of international OTPs](https://stytch.com/pricing) and understand how to protect your app against [toll fraud](https://stytch.com/docs/guides/passcodes/toll-fraud/overview).

        ### Next steps

        Collect the OTP which was delivered to the User. Call [Authenticate OTP](https://stytch.com/docs/api/authenticate-otp) using the OTP `code` along with the `phone_id` found in the response as the `method_id`.

        Fields:
          - phone_number: The phone number to use for one-time passcodes. The phone number should be in E.164 format (i.e. +1XXXXXXXXXX). You may use +10000000000 to test this endpoint, see [Testing](https://stytch.com/docs/home#resources_testing) for more detail.
          - expiration_minutes: Set the expiration for the one-time passcode, in minutes. The minimum expiration is 1 minute and the maximum is 10 minutes. The default expiration is 2 minutes.
          - attributes: Provided attributes to help with fraud detection. These values are pulled and passed into Stytch endpoints by your application.
          - create_user_as_pending: Flag for whether or not to save a user as pending vs active in Stytch. Defaults to false.
                If true, users will be saved with status pending in Stytch's backend until authenticated.
                If false, users will be created as active. An example usage of
                a true flag would be to require users to verify their phone by entering the OTP code before creating
                an account for them.
          - locale: Used to determine which language to use when sending the user this delivery method. Parameter is a [IETF BCP 47 language tag](https://www.w3.org/International/articles/language-tags/), e.g. `"en"`.

        Currently supported languages are English (`"en"`), Spanish (`"es"`), French (`"fr"`) and Brazilian Portuguese (`"pt-br"`); if no value is provided, the copy defaults to English.

        Request support for additional languages [here](https://docs.google.com/forms/d/e/1FAIpQLScZSpAu_m2AmLXRT3F3kap-s_mcV6UTBitYn6CdyWP0-o7YjQ/viewform?usp=sf_link")!

        """  # noqa
        headers: Dict[str, str] = {}
        data: Dict[str, Any] = {
            "phone_number": phone_number,
        }
        if expiration_minutes is not None:
            data["expiration_minutes"] = expiration_minutes
        if attributes is not None:
            data["attributes"] = (
                attributes if isinstance(attributes, dict) else attributes.dict()
            )
        if create_user_as_pending is not None:
            data["create_user_as_pending"] = create_user_as_pending
        if locale is not None:
            data["locale"] = locale

        url = self.api_base.url_for("/v1/otps/whatsapp/login_or_create", data)
        res = await self.async_client.post(url, data, headers)
        return LoginOrCreateResponse.from_json(res.response.status, res.json)



import time
from typing import Dict, Optional

from stytch.b2b.api.rbac import RBAC
from stytch.b2b.models.rbac import OrgPolicy, Policy
from stytch.shared.lazy_cache import LazyCache

DEFAULT_REFRESH_INTERVAL = 10 * 60  # 10 minutes


def _merge_policies(project_policy: Policy, org_policy: Optional[OrgPolicy]) -> Policy:
    """Merge project-level and organization-level policies.

    Organization policies supplement the project policy by adding additional roles.
    The merged policy combines roles from both, with org roles added to project roles.
    Resources and scopes come only from the project policy.
    """
    if org_policy is None:
        return project_policy

    return Policy(
        roles=list(project_policy.roles) + list(org_policy.roles),
        resources=project_policy.resources,
        scopes=project_policy.scopes,
    )


class _OrgPolicyCacheEntry:
    def __init__(self, org_policy: Optional[OrgPolicy], last_refresh_time: float):
        self.org_policy = org_policy
        self.last_refresh_time = last_refresh_time


class PolicyCache(LazyCache[Policy]):
    def __init__(
        self,
        rbac: RBAC,
        refresh_interval_seconds: int = DEFAULT_REFRESH_INTERVAL,
    ) -> None:
        self.rbac = rbac
        self.refresh_interval_seconds = refresh_interval_seconds
        self._org_cache: Dict[str, _OrgPolicyCacheEntry] = {}
        super().__init__(
            reload_func=self.reload_policy,
            async_reload_func=self.reload_policy_async,
            refresh_interval_seconds=refresh_interval_seconds,
        )

    def reload_policy(self, _: Optional[Policy]) -> Policy:
        resp = self.rbac.policy()
        assert resp.policy is not None
        return resp.policy

    async def reload_policy_async(self, _: Optional[Policy]) -> Policy:
        resp = await self.rbac.policy_async()
        assert resp.policy is not None
        return resp.policy

    def _org_needs_refresh(self, organization_id: str) -> bool:
        if organization_id not in self._org_cache:
            return True
        entry = self._org_cache[organization_id]
        return time.time() - entry.last_refresh_time > self.refresh_interval_seconds

    def _get_org_policy(self, organization_id: str) -> Optional[OrgPolicy]:
        """Get the organization policy, fetching and caching if needed."""
        if self._org_needs_refresh(organization_id):
            resp = self.rbac.organizations.get_org_policy(organization_id)
            self._org_cache[organization_id] = _OrgPolicyCacheEntry(
                org_policy=resp.org_policy,
                last_refresh_time=time.time(),
            )
        return self._org_cache[organization_id].org_policy

    async def _get_org_policy_async(self, organization_id: str) -> Optional[OrgPolicy]:
        """Get the organization policy, fetching and caching if needed (async)."""
        if self._org_needs_refresh(organization_id):
            resp = await self.rbac.organizations.get_org_policy_async(organization_id)
            self._org_cache[organization_id] = _OrgPolicyCacheEntry(
                org_policy=resp.org_policy,
                last_refresh_time=time.time(),
            )
        return self._org_cache[organization_id].org_policy

    def get_with_org(self, organization_id: str) -> Policy:
        """Get the project policy merged with the organization policy.

        This fetches both the project-level RBAC policy and the organization-specific
        policy, merging the org roles into the project policy for authorization checks.
        """
        project_policy = self.get()
        org_policy = self._get_org_policy(organization_id)
        return _merge_policies(project_policy, org_policy)

    async def get_with_org_async(self, organization_id: str) -> Policy:
        """Get the project policy merged with the organization policy (async).

        This fetches both the project-level RBAC policy and the organization-specific
        policy, merging the org roles into the project policy for authorization checks.
        """
        project_policy = await self.get_async()
        org_policy = await self._get_org_policy_async(organization_id)
        return _merge_policies(project_policy, org_policy)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

import datetime
import enum
from typing import Any, Dict, List, Optional

import pydantic

from stytch.core.response_base import ResponseBase


class SearchUsersQueryOperator(str, enum.Enum):
    OR = "OR"
    AND = "AND"


class BiometricRegistration(pydantic.BaseModel):
    """
    Fields:
      - biometric_registration_id: The unique ID for a biometric registration.
      - verified: The verified boolean denotes whether or not this send method, e.g. phone number, email address, etc., has been successfully authenticated by the User.
    """  # noqa

    biometric_registration_id: str
    verified: bool


class CryptoWallet(pydantic.BaseModel):
    """
    Fields:
      - crypto_wallet_id: The unique ID for a crypto wallet
      - crypto_wallet_address: The actual blockchain address of the User's crypto wallet.
      - crypto_wallet_type: The blockchain that the User's crypto wallet operates on, e.g. Ethereum, Solana, etc.
      - verified: The verified boolean denotes whether or not this send method, e.g. phone number, email address, etc., has been successfully authenticated by the User.
    """  # noqa

    crypto_wallet_id: str
    crypto_wallet_address: str
    crypto_wallet_type: str
    verified: bool


class Email(pydantic.BaseModel):
    """
    Fields:
      - email_id: The unique ID of a specific email address.
      - email: The email address.
      - verified: The verified boolean denotes whether or not this send method, e.g. phone number, email address, etc., has been successfully authenticated by the User.
    """  # noqa

    email_id: str
    email: str
    verified: bool


class Name(pydantic.BaseModel):
    """
    Fields:
      - first_name: The first name of the user.
      - middle_name: The middle name(s) of the user.
      - last_name: The last name of the user.
    """  # noqa

    first_name: Optional[str] = None
    middle_name: Optional[str] = None
    last_name: Optional[str] = None


class OAuthProvider(pydantic.BaseModel):
    """
    Fields:
      - provider_type: Denotes the OAuth identity provider that the user has authenticated with, e.g. Google, Facebook, GitHub etc.
      - provider_subject: The unique identifier for the User within a given OAuth provider. Also commonly called the "sub" or "Subject field" in OAuth protocols.
      - profile_picture_url: If available, the `profile_picture_url` is a url of the User's profile picture set in OAuth identity the provider that the User has authenticated with, e.g. Facebook profile picture.
      - locale: If available, the `locale` is the User's locale set in the OAuth identity provider that the user has authenticated with.
      - oauth_user_registration_id: The unique ID for an OAuth registration.
    """  # noqa

    provider_type: str
    provider_subject: str
    profile_picture_url: str
    locale: str
    oauth_user_registration_id: str


class Password(pydantic.BaseModel):
    """
    Fields:
      - password_id: The unique ID of a specific password
      - requires_reset: Indicates whether this password requires a password reset
    """  # noqa

    password_id: str
    requires_reset: bool


class PhoneNumber(pydantic.BaseModel):
    """
    Fields:
      - phone_id: The unique ID for the phone number.
      - phone_number: The phone number.
      - verified: The verified boolean denotes whether or not this send method, e.g. phone number, email address, etc., has been successfully authenticated by the User.
    """  # noqa

    phone_id: str
    phone_number: str
    verified: bool


class ResultsMetadata(pydantic.BaseModel):
    """
    Fields:
      - total: The total number of results returned by your search query. If totals have been disabled for your Stytch Workspace to improve search performance, the value will always be -1.
      - next_cursor: The `next_cursor` string is returned when your search result contains more than one page of results. This value is passed into your next search call in the `cursor` field.
    """  # noqa

    total: int
    next_cursor: Optional[str] = None


class SearchUsersQuery(pydantic.BaseModel):
    """
    Fields:
      - operator: The action to perform on the operands. The accepted values are:

      `AND` – all the operand values provided must match.

      `OR` – **[DEPRECATED]** the operator will return any matches to at least one of the operand values you supply. This parameter is retained for legacy use cases only and is no longer supported. We strongly recommend breaking down complex queries into multiple search queries instead.
      - operands: An array of operand objects that contains all of the filters and values to apply to your search search query.
    """  # noqa

    operator: SearchUsersQueryOperator
    operands: List[Dict[str, Any]]


class TOTP(pydantic.BaseModel):
    """
    Fields:
      - totp_id: The unique ID for a TOTP instance.
      - verified: The verified boolean denotes whether or not this send method, e.g. phone number, email address, etc., has been successfully authenticated by the User.
    """  # noqa

    totp_id: str
    verified: bool


class UserConnectedApp(pydantic.BaseModel):
    """
    Fields:
      - connected_app_id: The ID of the Connected App.
      - name: The name of the Connected App.
      - description: A description of the Connected App.
      - client_type: The type of Connected App. Supported values are `first_party`, `first_party_public`, `third_party`, and `third_party_public`.
      - scopes_granted: The scopes granted to the Connected App at the completion of the last authorization flow.
      - logo_url: The logo URL of the Connected App, if any.
    """  # noqa

    connected_app_id: str
    name: str
    description: str
    client_type: str
    scopes_granted: str
    logo_url: Optional[str] = None


class WebAuthnRegistration(pydantic.BaseModel):
    """
    Fields:
      - webauthn_registration_id: The unique ID for the Passkey or WebAuthn registration.
      - domain: The `domain` on which Passkey or WebAuthn registration was started. This will be the domain of your app.
      - user_agent: The user agent of the User.
      - verified: The verified boolean denotes whether or not this send method, e.g. phone number, email address, etc., has been successfully authenticated by the User.
      - authenticator_type: The `authenticator_type` string displays the requested authenticator type of the Passkey or WebAuthn device. The two valid types are "platform" and "cross-platform". If no value is present, the Passkey or WebAuthn device was created without an authenticator type preference.
      - name: The `name` of the Passkey or WebAuthn registration.
    """  # noqa

    webauthn_registration_id: str
    domain: str
    user_agent: str
    verified: bool
    authenticator_type: str
    name: str


class User(pydantic.BaseModel):
    """
    Fields:
      - user_id: The unique ID of the affected User.
      - emails: An array of email objects for the User.
      - status: The status of the User. The possible values are `pending` and `active`.
      - phone_numbers: An array of phone number objects linked to the User.
      - webauthn_registrations: An array that contains a list of all Passkey or WebAuthn registrations for a given User in the Stytch API.
      - providers: An array of OAuth `provider` objects linked to the User.
      - totps: An array containing a list of all TOTP instances for a given User in the Stytch API.
      - crypto_wallets: An array contains a list of all crypto wallets for a given User in the Stytch API.
      - biometric_registrations: An array that contains a list of all biometric registrations for a given User in the Stytch API.
      - is_locked: (no documentation yet)
      - roles: Roles assigned to this User.
       See the [RBAC guide](https://stytch.com/docs/guides/rbac/role-assignment) for more information about role assignment.
      - name: The name of the User. Each field in the `name` object is optional.
      - created_at: The timestamp of the User's creation. Values conform to the RFC 3339 standard and are expressed in UTC, e.g. `2021-12-29T12:33:09Z`.
      - password: The password object is returned for users with a password.
      - trusted_metadata: The `trusted_metadata` field contains an arbitrary JSON object of application-specific data. See the [Metadata](https://stytch.com/docs/api/metadata) reference for complete field behavior details.
      - untrusted_metadata: The `untrusted_metadata` field contains an arbitrary JSON object of application-specific data. Untrusted metadata can be edited by end users directly via the SDK, and **cannot be used to store critical information.** See the [Metadata](https://stytch.com/docs/api/metadata) reference for complete field behavior details.
      - external_id: (no documentation yet)
      - lock_created_at: (no documentation yet)
      - lock_expires_at: (no documentation yet)
    """  # noqa

    user_id: str
    emails: List[Email]
    status: str
    phone_numbers: List[PhoneNumber]
    webauthn_registrations: List[WebAuthnRegistration]
    providers: List[OAuthProvider]
    totps: List[TOTP]
    crypto_wallets: List[CryptoWallet]
    biometric_registrations: List[BiometricRegistration]
    is_locked: bool
    roles: List[str]
    name: Optional[Name] = None
    created_at: Optional[datetime.datetime] = None
    password: Optional[Password] = None
    trusted_metadata: Optional[Dict[str, Any]] = None
    untrusted_metadata: Optional[Dict[str, Any]] = None
    external_id: Optional[str] = None
    lock_created_at: Optional[datetime.datetime] = None
    lock_expires_at: Optional[datetime.datetime] = None


class ConnectedAppsResponse(ResponseBase):
    """Response type for `Users.connected_apps`.
    Fields:
      - connected_apps: An array of Connected Apps with which the User has successfully completed an authorization flow.
    """  # noqa

    connected_apps: List[UserConnectedApp]


class CreateResponse(ResponseBase):
    """Response type for `Users.create`.
    Fields:
      - user_id: The unique ID of the affected User.
      - email_id: The unique ID of a specific email address.
      - status: The status of the User. The possible values are `pending` and `active`.
      - phone_id: The unique ID for the phone number.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    email_id: str
    status: str
    phone_id: str
    user: User


class DeleteBiometricRegistrationResponse(ResponseBase):
    """Response type for `Users.delete_biometric_registration`.
    Fields:
      - user_id: The unique ID of the affected User.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    user: User


class DeleteCryptoWalletResponse(ResponseBase):
    """Response type for `Users.delete_crypto_wallet`.
    Fields:
      - user_id: The unique ID of the affected User.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    user: User


class DeleteEmailResponse(ResponseBase):
    """Response type for `Users.delete_email`.
    Fields:
      - user_id: The unique ID of the affected User.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    user: User


class DeleteExternalIdResponse(ResponseBase):
    user_id: str
    user: User


class DeleteOAuthRegistrationResponse(ResponseBase):
    """Response type for `Users.delete_oauth_registration`.
    Fields:
      - user_id: The unique ID of the affected User.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    user: User


class DeletePasswordResponse(ResponseBase):
    """Response type for `Users.delete_password`.
    Fields:
      - user_id: The unique ID of the affected User.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    user: User


class DeletePhoneNumberResponse(ResponseBase):
    """Response type for `Users.delete_phone_number`.
    Fields:
      - user_id: The unique ID of the affected User.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    user: User


class DeleteResponse(ResponseBase):
    """Response type for `Users.delete`.
    Fields:
      - user_id: The unique ID of the deleted User.
    """  # noqa

    user_id: str


class DeleteTOTPResponse(ResponseBase):
    """Response type for `Users.delete_totp`.
    Fields:
      - user_id: The unique ID of the affected User.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    user: User


class DeleteWebAuthnRegistrationResponse(ResponseBase):
    """Response type for `Users.delete_webauthn_registration`.
    Fields:
      - user_id: The unique ID of the affected User.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    user: User


class ExchangePrimaryFactorResponse(ResponseBase):
    """Response type for `Users.exchange_primary_factor`.
    Fields:
      - user_id: The unique ID of the affected User.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    user: User


class GetResponse(ResponseBase):
    """Response type for `Users.get`.
    Fields:
      - user_id: The unique ID of the returned User.
      - emails: An array of email objects for the User.
      - status: The status of the User. The possible values are `pending` and `active`.
      - phone_numbers: An array of phone number objects linked to the User.
      - webauthn_registrations: An array that contains a list of all Passkey or WebAuthn registrations for a given User in the Stytch API.
      - providers: An array of OAuth `provider` objects linked to the User.
      - totps: An array containing a list of all TOTP instances for a given User in the Stytch API.
      - crypto_wallets: An array contains a list of all crypto wallets for a given User in the Stytch API.
      - biometric_registrations: An array that contains a list of all biometric registrations for a given User in the Stytch API.
      - is_locked: (no documentation yet)
      - roles: Roles assigned to this User.
       See the [RBAC guide](https://stytch.com/docs/guides/rbac/role-assignment) for more information about role assignment.
      - name: The name of the User. Each field in the `name` object is optional.
      - created_at: The timestamp of the User's creation. Values conform to the RFC 3339 standard and are expressed in UTC, e.g. `2021-12-29T12:33:09Z`.
      - password: The password object is returned for users with a password.
      - trusted_metadata: The `trusted_metadata` field contains an arbitrary JSON object of application-specific data. See the [Metadata](https://stytch.com/docs/api/metadata) reference for complete field behavior details.
      - untrusted_metadata: The `untrusted_metadata` field contains an arbitrary JSON object of application-specific data. Untrusted metadata can be edited by end users directly via the SDK, and **cannot be used to store critical information.** See the [Metadata](https://stytch.com/docs/api/metadata) reference for complete field behavior details.
      - external_id: (no documentation yet)
      - lock_created_at: (no documentation yet)
      - lock_expires_at: (no documentation yet)
    """  # noqa

    user_id: str
    emails: List[Email]
    status: str
    phone_numbers: List[PhoneNumber]
    webauthn_registrations: List[WebAuthnRegistration]
    providers: List[OAuthProvider]
    totps: List[TOTP]
    crypto_wallets: List[CryptoWallet]
    biometric_registrations: List[BiometricRegistration]
    is_locked: bool
    roles: List[str]
    name: Optional[Name] = None
    created_at: Optional[datetime.datetime] = None
    password: Optional[Password] = None
    trusted_metadata: Optional[Dict[str, Any]] = None
    untrusted_metadata: Optional[Dict[str, Any]] = None
    external_id: Optional[str] = None
    lock_created_at: Optional[datetime.datetime] = None
    lock_expires_at: Optional[datetime.datetime] = None


class RevokeResponse(ResponseBase):
    """Response type for `Users.revoke`.
    Fields:
    """  # noqa


class SearchResponse(ResponseBase):
    """Response type for `Users.search`.
    Fields:
      - results: An array of results that match your search query.
      - results_metadata: The search `results_metadata` object contains metadata relevant to your specific query like total and `next_cursor`.
    """  # noqa

    results: List[User]
    results_metadata: ResultsMetadata


class UpdateResponse(ResponseBase):
    """Response type for `Users.update`.
    Fields:
      - user_id: The unique ID of the updated User.
      - emails: An array of email objects for the User.
      - phone_numbers: An array of phone number objects linked to the User.
      - crypto_wallets: An array contains a list of all crypto wallets for a given User in the Stytch API.
      - user: The `user` object affected by this API call. See the [Get user endpoint](https://stytch.com/docs/api/get-user) for complete response field details.
    """  # noqa

    user_id: str
    emails: List[Email]
    phone_numbers: List[PhoneNumber]
    crypto_wallets: List[CryptoWallet]
    user: User



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import List, Optional

from stytch.b2b.models.organizations import (
    GithubProviderInfo,
    HubspotProviderInfo,
    SlackProviderInfo,
)
from stytch.core.response_base import ResponseBase


class GithubResponse(ResponseBase):
    """Response type for `OAuthProviders.github`.
    Fields:
      - provider_type: Denotes the OAuth identity provider that the user has authenticated with, e.g. Google, Microsoft, GitHub etc.
      - registrations: A list of tokens the member is registered with.
    """  # noqa

    provider_type: str
    registrations: List[GithubProviderInfo]


class GoogleResponse(ResponseBase):
    """Response type for `OAuthProviders.google`.
    Fields:
      - provider_type: Denotes the OAuth identity provider that the user has authenticated with, e.g. Google, Microsoft, GitHub etc.
      - provider_subject: The unique identifier for the User within a given OAuth provider. Also commonly called the `sub` or "Subject field" in OAuth protocols.
      - id_token: The `id_token` returned by the OAuth provider. ID Tokens are JWTs that contain structured information about a user. The exact content of each ID Token varies from provider to provider. ID Tokens are returned from OAuth providers that conform to the [OpenID Connect](https://openid.net/foundation/) specification, which is based on OAuth.
      - scopes: The OAuth scopes included for a given provider. See each provider's section above to see which scopes are included by default and how to add custom scopes.
      - access_token: The `access_token` that you may use to access the User's data in the provider's API.
      - access_token_expires_in: The number of seconds until the access token expires.
      - refresh_token: The `refresh_token` that you may use to obtain a new `access_token` for the User within the provider's API.
    """  # noqa

    provider_type: str
    provider_subject: str
    id_token: str
    scopes: List[str]
    access_token: Optional[str] = None
    access_token_expires_in: Optional[int] = None
    refresh_token: Optional[str] = None


class HubspotResponse(ResponseBase):
    """Response type for `OAuthProviders.hubspot`.
    Fields:
      - provider_type: Denotes the OAuth identity provider that the user has authenticated with, e.g. Google, Microsoft, GitHub etc.
      - registrations: A list of tokens the member is registered with.
    """  # noqa

    provider_type: str
    registrations: List[HubspotProviderInfo]


class MicrosoftResponse(ResponseBase):
    """Response type for `OAuthProviders.microsoft`.
    Fields:
      - provider_type: Denotes the OAuth identity provider that the user has authenticated with, e.g. Google, Microsoft, GitHub etc.
      - provider_subject: The unique identifier for the User within a given OAuth provider. Also commonly called the `sub` or "Subject field" in OAuth protocols.
      - access_token: The `access_token` that you may use to access the User's data in the provider's API.
      - access_token_expires_in: The number of seconds until the access token expires.
      - id_token: The `id_token` returned by the OAuth provider. ID Tokens are JWTs that contain structured information about a user. The exact content of each ID Token varies from provider to provider. ID Tokens are returned from OAuth providers that conform to the [OpenID Connect](https://openid.net/foundation/) specification, which is based on OAuth.
      - scopes: The OAuth scopes included for a given provider. See each provider's section above to see which scopes are included by default and how to add custom scopes.
      - refresh_token: The `refresh_token` that you may use to obtain a new `access_token` for the User within the provider's API.
    """  # noqa

    provider_type: str
    provider_subject: str
    access_token: str
    access_token_expires_in: int
    id_token: str
    scopes: List[str]
    refresh_token: Optional[str] = None


class SlackResponse(ResponseBase):
    """Response type for `OAuthProviders.slack`.
    Fields:
      - provider_type: Denotes the OAuth identity provider that the user has authenticated with, e.g. Google, Microsoft, GitHub etc.
      - registrations: A list of tokens the member is registered with.
    """  # noqa

    provider_type: str
    registrations: List[SlackProviderInfo]



#!/usr/bin/env python3

import unittest
from test.constants import (
    TEST_CRYPTO_SIGNATURE,
    TEST_CRYPTO_WALLET_ADDRESS,
    TEST_CRYPTO_WALLET_TYPE,
    TEST_MAGIC_EMAIL,
    TEST_MAGIC_TOKEN,
    TEST_OAUTH_TOKEN,
    TEST_OTP_CODE,
    TEST_OTP_EMAIL,
    TEST_OTP_PHONE_NUMBER,
    TEST_PW_HASH,
    TEST_PW_HASH_TYPE,
    TEST_SESSION_TOKEN,
    TEST_TOTP_CODE,
    TEST_TOTP_RECOVERY_CODE,
    TEST_TOTP_USER_ID,
    TEST_USERS_NAME,
)
from test.integration_base import CreatedTestUser, IntegrationTestBase


class AsyncIntegrationTest(IntegrationTestBase, unittest.IsolatedAsyncioTestCase):
    async def test_crypto_wallets_async(self) -> None:
        api = self.b2c_client.crypto_wallets

        self.assertTrue(
            (
                await api.authenticate_start_async(
                    crypto_wallet_type=TEST_CRYPTO_WALLET_TYPE,
                    crypto_wallet_address=TEST_CRYPTO_WALLET_ADDRESS,
                )
            ).is_success
        )
        self.assertTrue(
            (
                await api.authenticate_async(
                    crypto_wallet_type=TEST_CRYPTO_WALLET_TYPE,
                    crypto_wallet_address=TEST_CRYPTO_WALLET_ADDRESS,
                    signature=TEST_CRYPTO_SIGNATURE,
                )
            ).is_success
        )

    async def test_magic_links_async(self) -> None:
        api = self.b2c_client.magic_links

        self.assertTrue(
            (await api.authenticate_async(token=TEST_MAGIC_TOKEN)).is_success
        )

        with self.skip_if_stytcherror(
            subtest_name="create_endpoint",
            skip_reason="Not approved for embedded magic links",
            error_message_pattern="requires approval before using",
        ):
            async with self._get_temporary_user_async() as user:
                assert isinstance(user, CreatedTestUser)
                self.assertTrue(
                    (await api.create_async(user_id=user.user_id)).is_success
                )

        with self.skip_if_stytcherror(
            subtest_name="email",
            skip_reason="No invite_redirect_url set up",
            error_message_pattern="There are no .* URLs registered",
        ):
            self.assertTrue(
                (await api.email.invite_async(email=TEST_MAGIC_EMAIL)).is_success
            )
            self.assertTrue(
                (await api.email.revoke_invite_async(email=TEST_MAGIC_EMAIL)).is_success
            )
            self.assertTrue(
                (
                    await api.email.login_or_create_async(email=TEST_MAGIC_EMAIL)
                ).is_success
            )
            self.assertTrue((await api.email.send_async(TEST_MAGIC_EMAIL)).is_success)

    async def test_oauth_async(self) -> None:
        api = self.b2c_client.oauth

        self.assertTrue(
            (await api.authenticate_async(token=TEST_OAUTH_TOKEN)).is_success
        )

    async def test_otp_async(self) -> None:
        api = self.b2c_client.otps

        with self.subTest("email"):
            self.assertTrue(
                (await api.email.login_or_create_async(email=TEST_OTP_EMAIL)).is_success
            )
            email_send_response = await api.email.send_async(email=TEST_OTP_EMAIL)
            self.assertTrue(email_send_response.is_success)
            self.assertTrue(
                (
                    await api.authenticate_async(
                        method_id=email_send_response.email_id, code=TEST_OTP_CODE
                    )
                ).is_success
            )
        with self.subTest("sms"):
            self.assertTrue(
                (
                    await api.sms.login_or_create_async(
                        phone_number=TEST_OTP_PHONE_NUMBER
                    )
                ).is_success
            )
            sms_send_response = await api.sms.send_async(
                phone_number=TEST_OTP_PHONE_NUMBER
            )
            self.assertTrue(sms_send_response.is_success)
            self.assertTrue(
                (
                    await api.authenticate_async(
                        method_id=sms_send_response.phone_id, code=TEST_OTP_CODE
                    )
                ).is_success
            )
        with self.subTest("whatsapp"):
            self.assertTrue(
                (
                    await api.sms.login_or_create_async(
                        phone_number=TEST_OTP_PHONE_NUMBER
                    )
                ).is_success
            )
            whatsapp_send_response = await api.whatsapp.send_async(
                phone_number=TEST_OTP_PHONE_NUMBER
            )
            self.assertTrue(whatsapp_send_response.is_success)
            self.assertTrue(
                (
                    await api.authenticate_async(
                        method_id=whatsapp_send_response.phone_id, code=TEST_OTP_CODE
                    )
                ).is_success
            )

    async def test_passwords_async(self) -> None:
        api = self.b2c_client.passwords

        async with self._get_temporary_user_async(create=False) as user:
            self.assertTrue(
                (await api.strength_check_async(password=user.old_password)).is_success
            )
            create_response = await api.create_async(
                email=user.email, password=user.old_password
            )
            self.assertTrue(create_response.is_success)
            authenticate_response = await api.authenticate_async(
                email=user.email, password=user.old_password
            )
            self.assertTrue(authenticate_response.is_success)
            # Remember to manually delete since we manually created!
            await self.b2c_client.users.delete_async(create_response.user_id)

        # Migrate an existing user created via magic link
        async with self._get_temporary_user_async(via_magic_link=True) as user:
            self.assertTrue(
                (
                    await api.migrate_async(
                        email=user.email,
                        hash=TEST_PW_HASH,
                        hash_type=TEST_PW_HASH_TYPE,
                    )
                ).is_success
            )

        with self.skip_if_stytcherror(
            subtest_name="email",
            skip_reason="No login_redirect_url set up",
            error_message_pattern="There are no .* URLs",
        ):
            # Can't test: there's no reset token that can be used for testing the API
            self.skipTest("No reset token to use for testing")
            async with self._get_temporary_user_async() as user:
                reset_start_response = await api.email.reset_start_async(
                    email=user.email
                )
                self.assertTrue(reset_start_response.is_success)
                # TODO: We don't have a sample reset token (see skipTest above)
                self.assertTrue(
                    (
                        await api.email.reset_async(
                            token="", password=user.new_password
                        )
                    ).is_success
                )
        with self.subTest("existing_password"):
            async with self._get_temporary_user_async() as user:
                self.assertTrue(
                    (
                        await api.existing_password.reset_async(
                            email=user.email,
                            existing_password=user.old_password,
                            new_password=user.new_password,
                        )
                    ).is_success
                )
        with self.subTest("session"):
            # Can't test: there's no sesison token that can be used for testing the API
            self.skipTest("No session token to use for testing")
            async with self._get_temporary_user_async() as user:
                # TODO: We don't have a sample session token (see skipTest above)
                self.assertTrue(
                    (
                        await api.sessions.reset_async(
                            session_token="",
                            password=user.new_password,
                        )
                    ).is_success
                )

    async def test_sessions_async(self) -> None:
        api = self.b2c_client.sessions

        async with self._get_temporary_user_async() as user:
            # TODO: With @overload, it should be possible to let
            # the type checker infer this statically. I tried, but
            # it was a bit complicated since it's not a simple type,
            # but actually returning a Generator from a @contextmanager
            assert isinstance(user, CreatedTestUser)
            self.assertTrue((await api.get_async(user_id=user.user_id)).is_success)
            self.assertTrue(
                (
                    await api.authenticate_async(session_token=TEST_SESSION_TOKEN)
                ).is_success
            )
            # Can't test revoke -- it doesn't support the TEST_SESSION_TOKEN
            # self.assertTrue(api.revoke(session_token=TEST_SESSION_TOKEN).is_success)
            self.assertTrue(
                (await api.get_jwks_async(project_id=self.project_id)).is_success
            )

    async def test_totps_async(self) -> None:
        api = self.b2c_client.totps

        self.assertTrue((await api.create_async(user_id=TEST_TOTP_USER_ID)).is_success)
        self.assertTrue(
            (
                await api.authenticate_async(
                    user_id=TEST_TOTP_USER_ID, totp_code=TEST_TOTP_CODE
                )
            ).is_success
        )
        self.assertTrue(
            (await api.recovery_codes_async(user_id=TEST_TOTP_USER_ID)).is_success
        )
        self.assertTrue(
            (
                await api.recover_async(
                    user_id=TEST_TOTP_USER_ID, recovery_code=TEST_TOTP_RECOVERY_CODE
                )
            ).is_success
        )

    async def test_users_async(self) -> None:
        # NOTE: the various `delete_XXX` can't easily be tested (yet)
        # since it would require we first create the user with each of those
        # methods/auth factors present (and possibly authenticated?).
        api = self.b2c_client.users

        async with self._get_temporary_user_async(create=False) as user:
            create_resp = await api.create_async(roles=[], email=user.email)
            self.assertTrue(create_resp.is_success)
            self.assertTrue((await api.search_async(limit=10)).is_success)
            self.assertTrue(
                (
                    await api.update_async(
                        user_id=create_resp.user_id,
                        name=TEST_USERS_NAME,
                    )
                ).is_success
            )
            self.assertTrue(
                (await api.get_async(user_id=create_resp.user_id)).is_success
            )
            self.assertTrue(
                (await api.delete_async(user_id=create_resp.user_id)).is_success
            )

    async def test_webauthn_async(self) -> None:
        api = self.b2c_client.webauthn
        # Can't test: webauthn requires calling browser functions
        # It would probably work if we had some way of using test API
        # sample values (like a sample public_key_credential for testing)
        self.skipTest("webauthn cannot be tested from this client")

        async with self._get_temporary_user_async() as user:
            assert isinstance(user, CreatedTestUser)
            # TODO: No test domain (see skipTest above)
            self.assertTrue(
                (
                    await api.register_start_async(user_id=user.user_id, domain="")
                ).is_success
            )
            # TODO: No test public key credential (see skipTest above)
            self.assertTrue(
                (
                    await api.register_async(
                        user_id=user.user_id,
                        public_key_credential="",
                    )
                ).is_success
            )
            # TODO: No test domain (see skipTest above)
            self.assertTrue(
                (
                    await api.authenticate_start_async(user_id=user.user_id, domain="")
                ).is_success
            )
            # TODO: No test public key credential (see skipTest above)
            self.assertTrue(
                (await api.authenticate_async(public_key_credential="")).is_success
            )

    async def test_authenticate(self) -> None:
        api = self.b2c_client.sessions

        async with self._get_temporary_user_async() as user:
            assert isinstance(user, CreatedTestUser)
            self.assertTrue(api.get(user_id=user.user_id).is_success)
            # Grab a recent JWT token and verify it's valid
            auth_response = api.authenticate(session_token=TEST_SESSION_TOKEN)
            response = await self.b2c_client.sessions.authenticate_jwt_async(
                session_jwt=auth_response.session_jwt
            )
            self.assertIsNotNone(response)
            if response is not None:
                self.assertEquals(auth_response.session_jwt, response.session_jwt)



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

import datetime
from typing import List, Optional

from stytch.consumer.models.fraud import Rule, RuleAction
from stytch.core.response_base import ResponseBase


class ListResponse(ResponseBase):
    """Response type for `Rules.list`.
    Fields:
      - next_cursor: The `next_cursor` string is returned when your result contains more than one page of results. This value is passed into your next request in the `cursor` field.
      - rules: A list of rules for the project
    """  # noqa

    next_cursor: str
    rules: List[Rule]


class SetResponse(ResponseBase):
    """Response type for `Rules.set`.
    Fields:
      - action: The action that will be returned for the specified identifier.
      - visitor_id: The visitor ID that a rule was set for.
      - browser_id: The browser ID that a rule was set for.
      - visitor_fingerprint: The visitor fingerprint that a rule was set for.
      - browser_fingerprint: The browser fingerprint that a rule was set for.
      - hardware_fingerprint: The hardware fingerprint that a rule was set for.
      - network_fingerprint: The network fingerprint that a rule was set for.
      - expires_at: The timestamp when the rule expires. Values conform to the RFC 3339 standard and are expressed in UTC, e.g. `2021-12-29T12:33:09Z`.
      - cidr_block: The CIDR block that a rule was set for. If an end user's IP address is within this CIDR block, this rule will be applied.
      - country_code: The country code that a rule was set for.
      - asn: The ASN that a rule was set for.
    """  # noqa

    action: RuleAction
    visitor_id: Optional[str] = None
    browser_id: Optional[str] = None
    visitor_fingerprint: Optional[str] = None
    browser_fingerprint: Optional[str] = None
    hardware_fingerprint: Optional[str] = None
    network_fingerprint: Optional[str] = None
    expires_at: Optional[datetime.datetime] = None
    cidr_block: Optional[str] = None
    country_code: Optional[str] = None
    asn: Optional[str] = None



# !!!
# WARNING: This file is autogenerated
# Only modify code within MANUAL() sections
# or your changes may be overwritten later!
# !!!

from __future__ import annotations

from typing import Dict, Optional

import pydantic

from stytch.core.response_base import ResponseBase
from stytch.shared.method_options import Authorization


class RevokeRequestOptions(pydantic.BaseModel):
    """
    Fields:
      - authorization: Optional authorization object.
    Pass in an active Stytch Member session token or session JWT and the request
    will be run using that member's permissions.
    """  # noqa

    authorization: Optional[Authorization] = None

    def add_headers(self, headers: Dict[str, str]) -> Dict[str, str]:
        if self.authorization is not None:
            headers = self.authorization.add_headers(headers)
        return headers


class RevokeResponse(ResponseBase):
    """Response type for `ConnectedApps.revoke`.
    Fields:
    """  # noqa
